from .okxclient import OkxClient
from .consts import *


class FundingAPI(OkxClient):


    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    # Get Non Tradable Assets
    def get_non_tradable_assets(self, ccy: str = ''):
        params = {'ccy': ccy}
        return self._request_with_params(GET, NON_TRADABLE_ASSETS, params)

    # Get Deposit Address
    def get_deposit_address(self, ccy):
        params = {'ccy': ccy}
        return self._request_with_params(GET, DEPOSIT_ADDRESS, params)

    # Get Transfer State
    def transfer_state(self, transId,type=''):
        params = {'transId': transId, 'type': type}
        return self._request_with_params(GET, TRANSFER_STATE, params)

    # Get Balance
    def get_balances(self, ccy=''):
        params = {'ccy': ccy}
        return self._request_with_params(GET, GET_BALANCES, params)

    # Get Account Configuration
    def funds_transfer(self, ccy, amt, from_, to, type='0', subAcct='', instId='', toInstId='',loanTrans=''):
        params = {'ccy': ccy, 'amt': amt, 'from': from_, 'to': to, 'type': type, 'subAcct': subAcct, 'instId': instId,
                  'toInstId': toInstId,'loanTrans':loanTrans}
        return self._request_with_params(POST, FUNDS_TRANSFER, params)

    # Withdrawal
    def withdrawal(self, ccy, amt, dest, toAddr, chain='', areaCode='', clientId=''):
        params = {'ccy': ccy, 'amt': amt, 'dest': dest, 'toAddr': toAddr, 'chain': chain,
                  'areaCode': areaCode, 'clientId': clientId}
        return self._request_with_params(POST, WITHDRAWAL_COIN, params)

    # Get Deposit History
    def get_deposit_history(self, ccy='', type='', state='', after='', before='', limit='', txId='', depId='', fromWdId=''):
        params = {'ccy': ccy, 'type': type, 'state': state, 'after': after, 'before': before, 'limit': limit, 'txId': txId,
                  'depId': depId, 'fromWdId': fromWdId}
        return self._request_with_params(GET, DEPOSIT_HISTORY, params)

    # Get Withdrawal History
    def get_withdrawal_history(self, ccy='', wdId='', state='', after='', before='', limit='',txId=''):
        params = {'ccy': ccy, 'wdId': wdId, 'state': state, 'after': after, 'before': before, 'limit': limit,'txId':txId}
        return self._request_with_params(GET, WITHDRAWAL_HISTORY, params)

    # Get Currencies
    def get_currencies(self, ccy=''):
        params = {'ccy': ccy}
        return self._request_with_params(GET, CURRENCY_INFO, params)

    # PiggyBank Purchase/Redemption
    def purchase_redempt(self, ccy, amt, side, rate):
        params = {'ccy': ccy, 'amt': amt, 'side': side,'rate':rate}
        return self._request_with_params(POST, PURCHASE_REDEMPT, params)

    # Get Withdrawal History
    def get_bills(self, ccy='', type='', after='', before='', limit=''):
        params = {'ccy': ccy, 'type': type, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, BILLS_INFO, params)


    #Get Deposit Lightning
    def get_deposit_lightning(self, ccy,amt,to=""):
        params = {'ccy':ccy,'amt':amt}
        if to:
            params = {'to':to}
        return self._request_with_params(GET, DEPOSIT_LIGHTNING, params)

    # Withdrawal Lightning
    def withdrawal_lightning(self, ccy,invoice,memo=''):
        params = {'ccy':ccy, 'invoice':invoice, 'memo':memo}
        return self._request_with_params(POST, WITHDRAWAL_LIGHTNING, params)

    #POST /api/v5/asset/cancel-withdrawal
    def cancel_withdrawal(self,wdId = ''):
        params = {
            'wdId':wdId
        }
        return self._request_with_params(POST, CANCEL_WITHDRAWAL, params)

    #POST /api/v5/asset/convert-dust-assets
    def convert_dust_assets(self,ccy = []):
        params = {
            'ccy':ccy
        }
        return self._request_with_params(POST, CONVERT_DUST_ASSETS, params)

    #GET /api/v5/asset/asset-valuation
    def get_asset_valuation(self,ccy = ''):
        params = {
            'ccy':ccy
        }
        return self._request_with_params(GET, ASSET_VALUATION, params)

    #Get non-tradable assets
    def get_non_tradable_assets(self, ccy=''):
        params = {
            'ccy': ccy
        }
        return self._request_with_params(GET, GET_NON_TRADABLE_ASSETS, params)

    #Get deposit withdraw status
    def get_deposit_withdraw_status(self, wdId='', txId='', ccy='', to='', chain=''):
        params = {'wdId': wdId, 'txId': txId, 'ccy': ccy, 'to': to, 'chain': chain}
        return self._request_with_params(GET, GET_DEPOSIT_WITHDrAW_STATUS, params)

    #Get withdrawal history
    def get_withdrawal_history(self, ccy='', wdId='', clientId='', txId='', type='', state='', after='', before	='', limit=''):
        params = {'ccy': ccy, 'wdId': wdId, 'clientId': clientId, 'txId': txId, 'type': type, 'state': state, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, GET_WITHDRAWAL_HISTORY, params)




from .BaseClient import OkxBaseClient
from ..constants import *


class RecurringBuyClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # - Place recurring buy order
    def place_recurring_buy_order(
        self,
        stgyName="",
        recurringList=[],
        period="",
        recurringDay="",
        recurringTime="",
        timeZone="",
        amt="",
        investmentCcy="",
        tdMode="",
        algoClOrdId="",
    ):
        params = {
            "stgyName": stgyName,
            "recurringList": recurringList,
            "period": period,
            "recurringDay": recurringDay,
            "recurringTime": recurringTime,
            "timeZone": timeZone,
            "amt": amt,
            "investmentCcy": investmentCcy,
            "tdMode": tdMode,
            "algoClOrdId": algoClOrdId,
            "tag": BROKER_ID,
        }
        return self._request(POST, RECURRING_ORDER_ALGO, params)

    # - Amend recurring buy order
    def amend_recurring_buy_order(self, algoId="", stgyName=""):
        params = {"algoId": algoId, "stgyName": stgyName}
        return self._request(POST, RECURRING_AMEND_ORDER_ALGO, params)

    # - Stop recurring buy order
    def stop_recurring_buy_order(self, orders_data):
        return self._request(POST, RECURRING_STOP_ORDER_ALGO, orders_data)

    # - Get recurring buy order list
    def get_recurring_buy_order_list(self, algoId="", after="", before="", limit=""):
        params = {"algoId": algoId, "after": after, "before": before, "limit": limit}
        return self._request(GET, RECURRING_ORDERS_ALGO_PENDING, params)

    # - Get recurring buy order history
    def get_recurring_buy_order_history(self, algoId="", after="", before="", limit=""):
        params = {"algoId": algoId, "after": after, "before": before, "limit": limit}
        return self._request(GET, RECURRING_ORDERS_ALGO_HISTORY, params)

    # - Get recurring buy order details
    def get_recurring_buy_order_details(self, algoId=""):
        params = {"algoId": algoId}
        return self._request(GET, RECURRING_ORDERS_ALGO_DETAILS, params)

    # - Get recurring buy sub orders
    def get_recurring_buy_sub_orders(
        self, algoId="", ordId="", after="", before="", limit=""
    ):
        params = {
            "algoId": algoId,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, RECURRING_SUB_ORDERS, params)



from .okxclient import OkxClient
from .consts import *


class GridAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    def grid_order_algo(self, instId='', algoOrdType='', maxPx='', minPx='', gridNum='', runType='', tpTriggerPx='',
                        slTriggerPx='', tag='', quoteSz='', baseSz='', sz='', direction='', lever='', basePos=''):
        params = {'instId': instId, 'algoOrdType': algoOrdType, 'maxPx': maxPx, 'minPx': minPx, 'gridNum': gridNum,
                  'runType': runType, 'tpTriggerPx': tpTriggerPx, 'slTriggerPx': slTriggerPx, 'tag': tag,
                  'quoteSz': quoteSz, 'baseSz': baseSz, 'sz': sz, 'direction': direction, 'lever': lever,
                  'basePos': basePos}
        return self._request_with_params(POST, GRID_ORDER_ALGO, params)

    def grid_amend_order_algo(self, algoId='', instId='', slTriggerPx='', tpTriggerPx=''):
        params = {'algoId': algoId, 'instId': instId, 'slTriggerPx': slTriggerPx, 'tpTriggerPx': tpTriggerPx}
        return self._request_with_params(POST, GRID_AMEND_ORDER_ALGO, params)

    def grid_stop_order_algo(self, algoId='', instId='', algoOrdType='', stopType=''):
        params = [{'algoId': algoId, 'instId': instId, 'algoOrdType': algoOrdType, 'stopType': stopType}]
        return self._request_with_params(POST, GRID_STOP_ORDER_ALGO, params)

    def grid_orders_algo_pending(self, algoOrdType='', algoId='', instId='', instType='', after='', before='',
                                 limit='', instFamily = ''):
        params = {'algoOrdType': algoOrdType, 'algoId': algoId, 'instId': instId, 'instType': instType, 'after': after,
                  'before': before, 'limit': limit,'instFamily':instFamily}
        return self._request_with_params(GET, GRID_ORDERS_ALGO_PENDING, params)

    def grid_orders_algo_history(self, algoOrdType='', algoId='', instId='', instType='', after='', before='',
                                 limit='',instFamily = ''):
        params = {'algoOrdType': algoOrdType, 'algoId': algoId, 'instId': instId, 'instType': instType, 'after': after,
                  'before': before, 'limit': limit,'instFamily':instFamily}
        return self._request_with_params(GET, GRID_ORDERS_ALGO_HISTORY, params)

    def grid_orders_algo_details(self, algoOrdType='', algoId=''):
        params = {'algoOrdType': algoOrdType, 'algoId': algoId}
        return self._request_with_params(GET, GRID_ORDERS_ALGO_DETAILS, params)

    def grid_sub_orders(self, algoId='', algoOrdType='', type='', groupId='', after='', before='', limit=''):
        params = {'algoId': algoId, 'algoOrdType': algoOrdType, 'type': type, 'groupId': groupId, 'after': after,
                  'before': before, 'limit': limit}
        return self._request_with_params(GET, GRID_SUB_ORDERS, params)

    def grid_positions(self, algoOrdType='', algoId=''):
        params = {'algoOrdType': algoOrdType, 'algoId': algoId}
        return self._request_with_params(GET, GRID_POSITIONS, params)

    def grid_withdraw_income(self, algoId=''):
        params = {'algoId': algoId}
        return self._request_with_params(POST, GRID_WITHDRAW_INCOME, params)

    def grid_compute_margin_balance(self, algoId='', type='', amt=''):
        params = {
            'algoId': algoId,
            'type': type,
            'amt': amt
        }
        return self._request_with_params(POST, GRID_COMPUTE_MARIGIN_BALANCE, params)

    def grid_adjust_margin_balance(self, algoId='', type='', amt='', percent=''):
        params = {
            'algoId': algoId,
            'type': type,
            'amt': amt,
            'percent': percent
        }
        return self._request_with_params(POST, GRID_MARGIN_BALANCE, params)

    def grid_ai_param(self, algoOrdType='', instId='', direction='', duration=''):
        params = {
            'algoOrdType': algoOrdType,
            'instId': instId,
            'direction': direction,
            'duration':duration
        }
        return self._request_with_params(GET, GRID_AI_PARAM, params)

    # - Place recurring buy order
    def place_recurring_buy_order(self, stgyName='', recurringList=[], period='', recurringDay='', recurringTime='',
                                  timeZone='', amt='', investmentCcy='', tdMode='', algoClOrdId='', tag=''):
        params = {'stgyName': stgyName, 'recurringList': recurringList, 'period': period, 'recurringDay': recurringDay,
                  'recurringTime': recurringTime,
                  'timeZone': timeZone, 'amt': amt, 'investmentCcy': investmentCcy, 'tdMode': tdMode,
                  'algoClOrdId': algoClOrdId, 'tag': tag}
        return self._request_with_params(POST, PLACE_RECURRING_BUY_ORDER, params)

    # - Amend recurring buy order
    def amend_recurring_buy_order(self, algoId='', stgyName=''):
        params = {'algoId': algoId, 'stgyName': stgyName}
        return self._request_with_params(POST, AMEND_RECURRING_BUY_ORDER, params)

    # - Stop recurring buy order
    def stop_recurring_buy_order(self, orders_data):
        return self._request_with_params(POST, STOP_RECURRING_BUY_ORDER, orders_data)

    # - Get recurring buy order list
    def get_recurring_buy_order_list(self, algoId='', after='', before='', limit=''):
        params = {
            'algoId': algoId,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_RECURRING_BUY_ORDER_LIST, params)

    # - Get recurring buy order history
    def get_recurring_buy_order_history(self, algoId='', after='', before='', limit=''):
        params = {
            'algoId': algoId,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_RECURRING_BUY_ORDER_HISTORY, params)

    # - Get recurring buy order details
    def get_recurring_buy_order_details(self, algoId=''):
        params = {'algoId': algoId}
        return self._request_with_params(GET, GET_RECURRING_BUY_ORDER_DETAILS, params)

    # - Get recurring buy sub orders
    def get_recurring_buy_sub_orders(self, algoId='', ordId='', after='', before='', limit=''):
        params = {
            'algoId': algoId,
            'ordId': ordId,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, GET_RECURRING_BUY_SUB_ORDERS, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class AlgoTradingClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Place Algo Order
    def place_algo_order(
        self,
        instId="",
        tdMode="",
        side="",
        ordType="",
        sz="",
        ccy="",
        posSide="",
        reduceOnly="",
        tpTriggerPx="",
        tpOrdPx="",
        slTriggerPx="",
        slOrdPx="",
        triggerPx="",
        orderPx="",
        tgtCcy="",
        pxVar="",
        pxSpread="",
        szLimit="",
        pxLimit="",
        timeInterval="",
        tpTriggerPxType="",
        slTriggerPxType="",
        callbackRatio="",
        callbackSpread="",
        activePx="",
        triggerPxType="",
        closeFraction="",
        quickMgnType="",
        algoClOrdId="",
    ):
        params = {
            "instId": instId,
            "tdMode": tdMode,
            "side": side,
            "ordType": ordType,
            "sz": sz,
            "ccy": ccy,
            "posSide": posSide,
            "reduceOnly": reduceOnly,
            "tpTriggerPx": tpTriggerPx,
            "tpOrdPx": tpOrdPx,
            "slTriggerPx": slTriggerPx,
            "slOrdPx": slOrdPx,
            "triggerPx": triggerPx,
            "orderPx": orderPx,
            "tgtCcy": tgtCcy,
            "pxVar": pxVar,
            "szLimit": szLimit,
            "pxLimit": pxLimit,
            "timeInterval": timeInterval,
            "pxSpread": pxSpread,
            "tpTriggerPxType": tpTriggerPxType,
            "slTriggerPxType": slTriggerPxType,
            "callbackRatio": callbackRatio,
            "callbackSpread": callbackSpread,
            "activePx": activePx,
            "tag": BROKER_ID,
            "triggerPxType": triggerPxType,
            "closeFraction": closeFraction,
            "quickMgnType": quickMgnType,
            "algoClOrdId": algoClOrdId,
        }
        return self._request(POST, TRADE_PLACE_ALGO_ORDER, params)

    # Cancel Algo Order
    def cancel_algo_order(self, params):
        return self._request(POST, TRADE_CANCEL_ALGOS, params)

    # Amend algo order
    def amend_algo_order(
        self,
        instId="",
        algoId="",
        algoClOrdId="",
        cxlOnFail="",
        reqId="",
        newSz="",
        newTpTriggerPx="",
        newTpOrdPx="",
        newSlTriggerPx="",
        newSlOrdPx="",
        newTpTriggerPxType="",
        newSlTriggerPxType="",
    ):
        params = {
            "instId": instId,
            "algoId": algoId,
            "algoClOrdId": algoClOrdId,
            "cxlOnFail": cxlOnFail,
            "reqId": reqId,
            "newSz": newSz,
            "newTpTriggerPx": newTpTriggerPx,
            "newTpOrdPx": newTpOrdPx,
            "newSlTriggerPx": newSlTriggerPx,
            "newSlOrdPx": newSlOrdPx,
            "newTpTriggerPxType": newTpTriggerPxType,
            "newSlTriggerPxType": newSlTriggerPxType,
        }
        return self._request(POST, TRADE_AMEND_ALGOS, params)

    # Cancel Advance Algos
    def cancel_advance_algos(self, params):
        return self._request(POST, TRADE_CANCEL_ADVANCE_ALGOS, params)

    # Get algo order details
    def get_algo_order_details(self, algoId="", algoClOrdId=""):
        params = {"algoId": algoId, "algoClOrdId": algoClOrdId}
        return self._request(GET, TRADE_GET_ALGO_ORDER, params)

    # Get Algo Order List
    def order_algos_list(
        self,
        ordType="",
        algoId="",
        instType="",
        instId="",
        after="",
        before="",
        limit="",
        algoClOrdId="",
    ):
        params = {
            "ordType": ordType,
            "algoId": algoId,
            "instType": instType,
            "instId": instId,
            "after": after,
            "before": before,
            "limit": limit,
            "algoClOrdId": algoClOrdId,
        }
        return self._request(GET, TRADE_ORDERS_ALGO_PENDING, params)

    # Get Algo Order History
    def order_algos_history(
        self,
        ordType,
        state="",
        algoId="",
        instType="",
        instId="",
        after="",
        before="",
        limit="",
    ):
        params = {
            "ordType": ordType,
            "state": state,
            "algoId": algoId,
            "instType": instType,
            "instId": instId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, TRADE_ORDERS_ALGO_HISTORY, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class SubAccountClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Get sub-account list
    def get_subaccount_list(self, enable="", subAcct="", after="", before="", limit=""):
        params = {
            "enable": enable,
            "subAcct": subAcct,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, USERS_SUBACCOUNT_LIST, params)

    # Reset the API Key of a sub-account
    def reset_subaccount_apikey(self, subAcct, apiKey, label="", perm="", ip=""):
        params = {"subAcct": subAcct, "apiKey": apiKey}

        if ip != "":
            params["ip"] = ip
        if label != "":
            params["label"] = label
        if perm != "":
            params["perm"] = perm
        return self._request(POST, USERS_SUBACCOUNT_MODIFY_APIKEY, params)

    # Get sub-account trading balance
    def get_trading_balance(self, subAcct):
        params = {"subAcct": subAcct}
        return self._request(GET, ACCOUNT_SUBACCOUNT_BALANCES, params)

    # Get sub-account funding balance
    def get_funding_balance(self, subAcct="", ccy=""):
        params = {"subAcct": subAcct, "ccy": ccy}
        return self._request(GET, ASSET_SUBACCOUNT_BALANCES, params)

    # Get sub-account maximum withdrawals
    def get_max_withdrawal(self, subAcct, ccy=""):
        params = {"subAcct": subAcct, "ccy": ccy}
        return self._request(GET, ACCOUNT_SUBACCOUNT_MAX_WITHDRAWAL, params)

    # Get history of sub-account transfer
    def get_transfer_history(
        self, ccy="", type="", subAcct="", after="", before="", limit=""
    ):
        params = {
            "ccy": ccy,
            "type": type,
            "subAcct": subAcct,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ASSET_SUBACCOUNT_BILLS, params)

    # Get history of managed sub-account transfer
    def get_managed_transfer_history(
        self, ccy="", type="", subAcct="", subUid="", after="", before="", limit=""
    ):
        params = {
            "ccy": ccy,
            "type": type,
            "subAcct": subAcct,
            "subUid": subUid,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, ASSET_SUBACCOUNT_MANAGED_SUBACCOUNT_BILLS, params)

    # Master accounts manage the transfers between sub-accounts
    def transfer_between_sub_accounts(
        self,
        ccy,
        amt,
        froms,
        to,
        fromSubAccount,
        toSubAccount,
        loanTrans="false",
        omitPosRisk="false",
    ):
        params = {
            "ccy": ccy,
            "amt": amt,
            "from": froms,
            "to": to,
            "fromSubAccount": fromSubAccount,
            "toSubAccount": toSubAccount,
            "loanTrans": loanTrans,
            "omitPosRisk": omitPosRisk,
        }
        return self._request(POST, ASSET_SUBACCOUNT_TRANSFER, params)

    # Set permission of transfer out
    def set_permission_transfer_out(self, subAcct="", canTransOut=""):
        params = {"subAcct": subAcct, "canTransOut": canTransOut}
        return self._request(POST, USERS_SUBACCOUNT_SET_TRANSFER_OUT, params)

    # Get custody trading sub-account list
    def get_entrust_subaccount_list(self, subAcct=""):
        params = {"subAcct": subAcct}
        return self._request(GET, USERS_ENTRUST_SUBACCOUNT_LIST, params)

    # Set sub-accounts VIP loan allocation
    def set_sub_accounts_VIP_loan(self, enable="", alloc=[]):
        params = {"enable": enable, "alloc": alloc}
        return self._request(POST, ACCOUNT_SUBACCOUNT_SET_LOAN_ALLOCATION, params)

    # Get sub_account borrow interest and limit
    def get_sub_account_borrow_interest_and_limit(self, subAcct="", ccy=""):
        params = {"subAcct": subAcct, "ccy": ccy}
        return self._request(GET, ACCOUNT_SUBACCOUNT_INTEREST_LIMITS, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class CopyTradingClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Get existing leading positions
    def get_existing_leading_positions(self, instId=""):
        params = {"instId": instId}
        return self._request(GET, COPYTRADING_CURRENT_SUBPOSITIONS, params)

    # Get leading position history
    def get_leading_position_history(self, instId="", after="", before="", limit=""):
        params = {"instId": instId, "after": after, "before": before, "limit": limit}
        return self._request(GET, COPYTRADING_SUBPOSITIONS_HISTORY, params)

    # Place leading stop order
    def place_leading_stop_order(
        self,
        subPosId="",
        tpTriggerPx="",
        slTriggerPx="",
        tpTriggerPxType="",
        slTriggerPxType="",
    ):
        params = {
            "subPosId": subPosId,
            "tpTriggerPx": tpTriggerPx,
            "slTriggerPx": slTriggerPx,
            "tpTriggerPxType": tpTriggerPxType,
            "slTriggerPxType": slTriggerPxType,
        }
        return self._request(POST, COPYTRADING_ALGO_ORDER, params)

    # Close leading position
    def close_leading_position(self, subPosId=""):
        params = {"subPosId": subPosId}
        return self._request(POST, COPYTRADING_CLOSE_SUBPOSITION, params)

    # Get leading instruments
    def get_leading_instruments(self):
        return self._request(GET, COPYTRADING_INSTRUMENTS)

    # Amend leading instruments
    def amend_leading_instruments(self, instId=""):
        params = {"instId": instId}
        return self._request(POST, COPYTRADING_SET_INSTRUMENTS, params)

    # Get profit sharing details
    def get_profit_sharing_details(self, after="", before="", limit=""):
        params = {"after": after, "before": before, "limit": limit}
        return self._request(GET, COPYTRADING_PROFIT_SHARING_DETAILS, params)

    # Get total profit sharing
    def get_total_profit_sharing(self):
        return self._request(GET, COPYTRADING_TOTAL_PROFIT_SHARING)

    # Get unrealized profit sharing details
    def get_unrealized_profit_sharing_details(self):
        return self._request(GET, COPYTRADING_UNREALIZED_PROFIT_SHARING_DETAILS)

    # Get Total unrealized profit sharing
    def get_total_unrealized_profit_sharing(self, instType=""):
        params = {
            "instType": instType,
        }
        return self._request(GET, COPYTRADING_TOTAL_UNREALIZED_PROFIT_SHARING, params)

    # Apply for lead trading
    def apply_lead_trading(self, instId, instType=""):
        params = {
            "instId": instId,
            "instType": instType,
        }
        return self._request(POST, COPYTRADING_APPLY_LEAD_TRADING, params)

    # Stop lead trading
    def stop_lead_trading(self, instType=""):
        params = {
            "instType": instType,
        }
        return self._request(POST, COPYTRADING_STOP_LEAD_TRADING, params)

    # Amend profit sharing ratio
    def amend_profit_sharing_ratio(self, profitSharingRatio, instType=""):
        params = {
            "instType": instType,
            "profitSharingRatio": profitSharingRatio,
        }
        return self._request(POST, COPYTRADING_AMEND_PROFIT_SHARING_RATIO, params)

    # Get Account configuration
    def get_account_configuration(self):
        return self._request(GET, COPYTRADING_CONFIG)

    # First copy settings
    def first_copy_settings(
        self,
        instType,
        uniqueCode,
        copyMgnMode,
        copyInstIdType,
        instId="",
        copyMode="",
        copyTotalAmt="",
        copyAmt="",
        copyRatio="",
        tpRatio="",
        slRatio="",
        slTotalAmt="",
        subPosCloseType="",
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "copyMgnMode": copyMgnMode,
            "copyInstIdType": copyInstIdType,
            "instId": instId,
            "copyMode": copyMode,
            "copyTotalAmt": copyTotalAmt,
            "copyAmt": copyAmt,
            "copyRatio": copyRatio,
            "tpRatio": tpRatio,
            "slRatio": slRatio,
            "slTotalAmt": slTotalAmt,
            "subPosCloseType": subPosCloseType,
        }
        return self._request(POST, COPYTRADING_FIRST_COPY_SETTINGS, params)

    # Amend copy settings
    def amend_copy_settings(
        self,
        instType,
        uniqueCode,
        copyMgnMode,
        copyInstIdType,
        instId="",
        copyMode="",
        copyTotalAmt="",
        copyAmt="",
        copyRatio="",
        tpRatio="",
        slRatio="",
        slTotalAmt="",
        subPosCloseType="",
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "copyMgnMode": copyMgnMode,
            "copyInstIdType": copyInstIdType,
            "instId": instId,
            "copyMode": copyMode,
            "copyTotalAmt": copyTotalAmt,
            "copyAmt": copyAmt,
            "copyRatio": copyRatio,
            "tpRatio": tpRatio,
            "slRatio": slRatio,
            "slTotalAmt": slTotalAmt,
            "subPosCloseType": subPosCloseType,
        }
        return self._request(POST, COPYTRADING_AMEND_COPY_SETTINGS, params)

    # Stop copying
    def stop_copying(self, instType, uniqueCode, subPosCloseType):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "subPosCloseType": subPosCloseType,
        }
        return self._request(POST, COPYTRADING_STOP_COPY_TRADING, params)

    # Get copy settings
    def get_copy_settings(self, instType, uniqueCode):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
        }
        return self._request(GET, COPYTRADING_COPY_SETTINGS, params)

    # Get Multiple leverages
    def get_multiple_leverages(self, mgnMode, uniqueCode, instId=""):
        params = {
            "mgnMode": mgnMode,
            "uniqueCode": uniqueCode,
            "instId": instId,
        }
        return self._request(GET, COPYTRADING_BATCH_LEVERAGE_INFO, params)

    # Set Multiple leverages
    def set_multiple_leverages(self, mgnMode, lever, instId=""):
        params = {
            "mgnMode": mgnMode,
            "lever": lever,
            "instId": instId,
        }
        return self._request(POST, COPYTRADING_BATCH_SET_LEVERAGE, params)

    # Get my lead traders
    def get_my_lead_traders(self, instType=""):
        params = {
            "instType": instType,
        }
        return self._request(GET, COPYTRADING_CURRENT_LEAD_TRADERS, params)

    # Get my history lead traders
    def get_my_history_lead_traders(self, instType="", after="", before="", limit=""):
        params = {
            "instType": instType,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_LEAD_TRADERS_HISTORY, params)

    # Get Copy trading configuration
    def get_public_config(self, instType=""):
        params = {
            "instType": instType,
        }
        return self._request(GET, COPYTRADING_PUBLIC_CONFIG, params)

    # Get Lead trader ranks
    def get_public_lead_traders(
        self,
        instType="",
        sortType="",
        state="",
        minLeadDays="",
        minAssets="",
        maxAssets="",
        minAum="",
        maxAum="",
        dataVer="",
        page="",
        limit="",
    ):
        params = {
            "instType": instType,
            "sortType": sortType,
            "state": state,
            "minLeadDays": minLeadDays,
            "minAssets": minAssets,
            "maxAssets": maxAssets,
            "minAum": minAum,
            "maxAum": maxAum,
            "dataVer": dataVer,
            "dataVer": dataVer,
            "page": page,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PUBLIC_LEAD_TRADERS, params)

    # Get Lead trader weekly pnl
    def get_public_weekly_pnl(self, instType, uniqueCode):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
        }
        return self._request(GET, COPYTRADING_PUBLIC_WEEKLY_PNL, params)

    # Get Lead trader daily pnl
    def get_public_daily_pnl(self, instType, uniqueCode, lastDays):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "lastDays": lastDays,
        }
        return self._request(GET, COPYTRADING_PUBLIC_PNL, params)

    # Get Lead trader stats
    def get_public_stats(self, instType, uniqueCode, lastDays):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "lastDays": lastDays,
        }
        return self._request(GET, COPYTRADING_PUBLIC_STATS, params)

    # Get Lead trader currency preferences
    def get_public_preference_currency(self, instType, uniqueCode):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
        }
        return self._request(GET, COPYTRADING_PUBLIC_PREFERENCE_CURRENCY, params)

    # Get Lead trader current lead positions
    def get_public_current_subpositions(
        self, instType, uniqueCode, after="", before="", limit=""
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PUBLIC_CURRENT_SUBPOSITIONS, params)

    # Get Lead trader lead position history
    def get_public_subpositions_history(
        self, instType, uniqueCode, after="", before="", limit=""
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PUBLIC_SUBPOSITIONS_HISTORY, params)

    # Get Copy traders
    def get_public_copy_traders(self, instType, uniqueCode, limit=""):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PUBLIC_COPY_TRADERS, params)

    # Get Lead trader ranks (private)
    def get_lead_traders(
        self,
        instType="",
        sortType="",
        state="",
        minLeadDays="",
        minAssets="",
        maxAssets="",
        minAum="",
        maxAum="",
        dataVer="",
        page="",
        limit="",
    ):
        params = {
            "instType": instType,
            "sortType": sortType,
            "state": state,
            "minLeadDays": minLeadDays,
            "minAssets": minAssets,
            "maxAssets": maxAssets,
            "minAum": minAum,
            "maxAum": maxAum,
            "dataVer": dataVer,
            "page": page,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_LEAD_TRADERS, params)

    # Get Lead trader weekly pnl (private)
    def get_weekly_pnl(self, instType, uniqueCode):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
        }
        return self._request(GET, COPYTRADING_WEEKLY_PNL, params)

    # Get Lead trader daily pnl (private)
    def get_daily_pnl(self, instType, uniqueCode, lastDays):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "lastDays": lastDays,
        }
        return self._request(GET, COPYTRADING_PNL, params)

    # Get Lead trader stats (private)
    def get_stats(self, instType, uniqueCode, lastDays):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "lastDays": lastDays,
        }
        return self._request(GET, COPYTRADING_STATS, params)

    # Get Lead trader currency preferences (private)
    def get_preference_currency(self, instType, uniqueCode):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
        }
        return self._request(GET, COPYTRADING_PREFERENCE_CURRENCY, params)

    # Get Lead trader current lead positions (private)
    def get_performance_current_subpositions(
        self, instType, uniqueCode, after="", before="", limit=""
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PRIVATE_CURRENT_SUBPOSITIONS, params)

    # Get Lead trader lead position history (private)
    def get_performance_subpositions_history(
        self, instType, uniqueCode, after="", before="", limit=""
    ):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_PERFORMANCE_SUBPOSITIONS_HISTORY, params)

    # Get Copy traders (private)
    def get_copy_traders(self, instType, uniqueCode, limit=""):
        params = {
            "instType": instType,
            "uniqueCode": uniqueCode,
            "limit": limit,
        }
        return self._request(GET, COPYTRADING_COPY_TRADERS, params)



# Http header
API_URL = 'https://www.okx.com'

CONTENT_TYPE = 'Content-Type'
OK_ACCESS_KEY = 'OK-ACCESS-KEY'
OK_ACCESS_SIGN = 'OK-ACCESS-SIGN'
OK_ACCESS_TIMESTAMP = 'OK-ACCESS-TIMESTAMP'
OK_ACCESS_PASSPHRASE = 'OK-ACCESS-PASSPHRASE'

ACEEPT = 'Accept'
COOKIE = 'Cookie'
LOCALE = 'Locale='

APPLICATION_JSON = 'application/json'

GET = "GET"
POST = "POST"

SERVER_TIMESTAMP_URL = '/api/v5/public/time'

# Account
POSITION_RISK='/api/v5/account/account-position-risk'
ACCOUNT_INFO = '/api/v5/account/balance'
POSITION_INFO = '/api/v5/account/positions'
BILLS_DETAIL = '/api/v5/account/bills'
BILLS_ARCHIVE = '/api/v5/account/bills-archive'
ACCOUNT_CONFIG = '/api/v5/account/config'
POSITION_MODE = '/api/v5/account/set-position-mode'
SET_LEVERAGE = '/api/v5/account/set-leverage'
MAX_TRADE_SIZE = '/api/v5/account/max-size'
MAX_AVAIL_SIZE = '/api/v5/account/max-avail-size'
ADJUSTMENT_MARGIN = '/api/v5/account/position/margin-balance'
GET_LEVERAGE = '/api/v5/account/leverage-info'
MAX_LOAN = '/api/v5/account/max-loan'
FEE_RATES = '/api/v5/account/trade-fee'
INTEREST_ACCRUED = '/api/v5/account/interest-accrued'
INTEREST_RATE = '/api/v5/account/interest-rate'
SET_GREEKS = '/api/v5/account/set-greeks'
ISOLATED_MODE = '/api/v5/account/set-isolated-mode'
MAX_WITHDRAWAL = '/api/v5/account/max-withdrawal'
ACCOUNT_RISK = '/api/v5/account/risk-state' #need add
BORROW_REPAY = '/api/v5/account/borrow-repay'
BORROW_REPAY_HISTORY = '/api/v5/account/borrow-repay-history'
INTEREST_LIMITS = '/api/v5/account/interest-limits'
SIMULATED_MARGIN = '/api/v5/account/simulated_margin'
GREEKS = '/api/v5/account/greeks'
POSITIONS_HISTORY = '/api/v5/account/positions-history'
GET_PM_LIMIT = '/api/v5/account/position-tiers'
GET_VIP_INTEREST_ACCRUED_DATA = '/api/v5/account/vip-interest-accrued'
GET_VIP_INTEREST_DEDUCTED_DATA = '/api/v5/account/vip-interest-deducted'
GET_VIP_LOAN_ORDER_LIST= '/api/v5/account/vip-loan-order-list'
GET_VIP_LOAN_ORDER_DETAIL= '/api/v5/account/vip-loan-order-detail'
SET_RISK_OFFSET_TYPE = '/api/v5/account/set-riskOffset-type'
SET_AUTO_LOAN = '/api/v5/account/set-auto-loan'
SET_ACCOUNT_LEVEL = '/api/v5/account/set-account-level'
ACTIVSTE_OPTION = '/api/v5/account/activate-option'
POSITION_BUILDER = '/api/v5/account/position-builder'
GET_INSTRUMENTS = '/api/v5/account/instruments'
BORROWING_LIMIT = '/api/v5/account/fixed-loan/borrowing-limit'
BORROWING_QUOTE = '/api/v5/account/fixed-loan/borrowing-quote'
PLACE_BORROWING_ORDER= '/api/v5/account/fixed-loan/borrowing-order'
AMEND_BORROWING_ORDER='/api/v5/account/fixed-loan/amend-borrowing-order'
MANUAL_REBORROW = '/api/v5/account/fixed-loan/manual-reborrow'
REPAY_BORROWING_ORDER='/api/v5/account/fixed-loan/repay-borrowing-order'
BORROWING_ORDERS_LIST='/api/v5/account/fixed-loan/borrowing-orders-list'
MANUAL_REBORROW_REPAY = '/api/v5/account/spot-manual-borrow-repay'
SET_AUTO_REPAY='/api/v5/account/set-auto-repay'
GET_BORROW_REPAY_HISTORY='/api/v5/account/spot-borrow-repay-history'

# Funding
NON_TRADABLE_ASSETS = '/api/v5/asset/non-tradable-assets'
DEPOSIT_ADDRESS = '/api/v5/asset/deposit-address'
GET_BALANCES = '/api/v5/asset/balances'
FUNDS_TRANSFER = '/api/v5/asset/transfer'
TRANSFER_STATE = '/api/v5/asset/transfer-state'
WITHDRAWAL_COIN = '/api/v5/asset/withdrawal'
DEPOSIT_HISTORY = '/api/v5/asset/deposit-history'
CURRENCY_INFO = '/api/v5/asset/currencies'
PURCHASE_REDEMPT = '/api/v5/asset/purchase_redempt'
BILLS_INFO = '/api/v5/asset/bills'
DEPOSIT_LIGHTNING = '/api/v5/asset/deposit-lightning'
WITHDRAWAL_LIGHTNING = '/api/v5/asset/withdrawal-lightning'
CANCEL_WITHDRAWAL = '/api/v5/asset/cancel-withdrawal'
WITHDRAWAL_HISTORY = '/api/v5/asset/withdrawal-history'
CONVERT_DUST_ASSETS = '/api/v5/asset/convert-dust-assets'
ASSET_VALUATION = '/api/v5/asset/asset-valuation'
GET_WITHDRAWAL_HISTORY = '/api/v5/asset/withdrawal-history'
GET_NON_TRADABLE_ASSETS = '/api/v5/asset/non-tradable-assets'
GET_DEPOSIT_WITHDrAW_STATUS = '/api/v5/asset/deposit-withdraw-status'


# Market Data
TICKERS_INFO = '/api/v5/market/tickers'
TICKER_INFO = '/api/v5/market/ticker'
INDEX_TICKERS = '/api/v5/market/index-tickers'
ORDER_BOOKS = '/api/v5/market/books'
MARKET_CANDLES = '/api/v5/market/candles'
HISTORY_CANDLES = '/api/v5/market/history-candles'
INDEX_CANSLES = '/api/v5/market/index-candles'
MARKPRICE_CANDLES = '/api/v5/market/mark-price-candles'
MARKET_TRADES = '/api/v5/market/trades'
VOLUMNE = '/api/v5/market/platform-24-volume'
INDEX_COMPONENTS = '/api/v5/market/index-components'
EXCHANGE_RATE = '/api/v5/market/exchange-rate'
HISTORY_TRADES = '/api/v5/market/history-trades'
BLOCK_TICKERS = '/api/v5/market/block-tickers'
BLOCK_TICKER = '/api/v5/market/block-ticker'
BLOCK_TRADES = '/api/v5/market/block-trades'
GET_ORDER_LITE_BOOK = '/api/v5/market/books-lite'
GET_OPTION_TRADES = '/api/v5/market/option/instrument-family-trades'

# Public Data
INSTRUMENT_INFO = '/api/v5/public/instruments'
DELIVERY_EXERCISE = '/api/v5/public/delivery-exercise-history'
OPEN_INTEREST = '/api/v5/public/open-interest'
FUNDING_RATE = '/api/v5/public/funding-rate'
FUNDING_RATE_HISTORY = '/api/v5/public/funding-rate-history'
PRICE_LIMIT = '/api/v5/public/price-limit'
OPT_SUMMARY = '/api/v5/public/opt-summary'
ESTIMATED_PRICE = '/api/v5/public/estimated-price'
DICCOUNT_INTETEST_INFO = '/api/v5/public/discount-rate-interest-free-quota'
SYSTEM_TIME = '/api/v5/public/time'
LIQUIDATION_ORDERS = '/api/v5/public/liquidation-orders'
MARK_PRICE = '/api/v5/public/mark-price'
TIER = '/api/v5/public/position-tiers'
INTEREST_LOAN = '/api/v5/public/interest-rate-loan-quota'
UNDERLYING = '/api/v5/public/underlying'
VIP_INTEREST_RATE_LOAN_QUOTA = '/api/v5/public/vip-interest-rate-loan-quota'
INSURANCE_FUND = '/api/v5/public/insurance-fund'
CONVERT_CONTRACT_COIN = '/api/v5/public/convert-contract-coin'
GET_OPTION_TICKBANDS = '/api/v5/public/instrument-tick-bands'
GET_OPTION_TRADES = '/api/v5/public/option-trades'

# Trading data
SUPPORT_COIN = '/api/v5/rubik/stat/trading-data/support-coin'
TAKER_VOLUME = '/api/v5/rubik/stat/taker-volume'
MARGIN_LENDING_RATIO = '/api/v5/rubik/stat/margin/loan-ratio'
LONG_SHORT_RATIO = '/api/v5/rubik/stat/contracts/long-short-account-ratio'
CONTRACTS_INTEREST_VOLUME = '/api/v5/rubik/stat/contracts/open-interest-volume'
OPTIONS_INTEREST_VOLUME = '/api/v5/rubik/stat/option/open-interest-volume'
PUT_CALL_RATIO = '/api/v5/rubik/stat/option/open-interest-volume-ratio'
OPEN_INTEREST_VOLUME_EXPIRY = '/api/v5/rubik/stat/option/open-interest-volume-expiry'
INTEREST_VOLUME_STRIKE = '/api/v5/rubik/stat/option/open-interest-volume-strike'
TAKER_FLOW = '/api/v5/rubik/stat/option/taker-block-volume'

# Trade
PLACR_ORDER = '/api/v5/trade/order'
BATCH_ORDERS = '/api/v5/trade/batch-orders'
CANCEL_ORDER = '/api/v5/trade/cancel-order'
CANCEL_BATCH_ORDERS = '/api/v5/trade/cancel-batch-orders'
AMEND_ORDER = '/api/v5/trade/amend-order'
AMEND_BATCH_ORDER = '/api/v5/trade/amend-batch-orders'
CLOSE_POSITION = '/api/v5/trade/close-position'
ORDER_INFO = '/api/v5/trade/order'
ORDERS_PENDING = '/api/v5/trade/orders-pending'
ORDERS_HISTORY = '/api/v5/trade/orders-history'
ORDERS_HISTORY_ARCHIVE = '/api/v5/trade/orders-history-archive'
ORDER_FILLS = '/api/v5/trade/fills'
ORDERS_FILLS_HISTORY = '/api/v5/trade/fills-history'
PLACE_ALGO_ORDER = '/api/v5/trade/order-algo'
CANCEL_ALGOS = '/api/v5/trade/cancel-algos'
ORDERS_ALGO_PENDING = '/api/v5/trade/orders-algo-pending'
ORDERS_ALGO_HISTORY = '/api/v5/trade/orders-algo-history'
GET_ALGO_ORDER_DETAILS = '/api/v5/trade/order-algo'
AMEND_ALGO_ORDER = '/api/v5/trade/amend-algos'
EASY_CONVERT_CURRENCY_LIST = '/api/v5/trade/easy-convert-currency-list'
EASY_CONVERT = '/api/v5/trade/easy-convert'
CONVERT_EASY_HISTORY = '/api/v5/trade/easy-convert-history'
ONE_CLICK_REPAY_SUPPORT = '/api/v5/trade/one-click-repay-currency-list'
ONE_CLICK_REPAY = '/api/v5/trade/one-click-repay'
ONE_CLICK_REPAY_HISTORY = '/api/v5/trade/one-click-repay-history'
ONE_CLICK_REPAY_SUPPORT_V2 = '/api/v5/trade/one-click-repay-currency-list-v2'
ONE_CLICK_REPAY_V2 = '/api/v5/trade/one-click-repay-v2'
ONE_CLICK_REPAY_HISTORY_V2 = '/api/v5/trade/one-click-repay-history-v2'


# SubAccount
BALANCE = '/api/v5/account/subaccount/balances'
BILLs = '/api/v5/asset/subaccount/bills'
RESET = '/api/v5/users/subaccount/modify-apikey'
VIEW_LIST = '/api/v5/users/subaccount/list'
SUBACCOUNT_TRANSFER = '/api/v5/asset/subaccount/transfer'
ENTRUST_SUBACCOUNT_LIST = '/api/v5/users/entrust-subaccount-list'
SET_TRSNSFER_OUT = '/api/v5/users/subaccount/set-transfer-out'
GET_ASSET_SUBACCOUNT_BALANCE = '/api/v5/asset/subaccount/balances'
SET_SUB_ACCOUNTS_VIP_LOAN = '/api/v5/account/subaccount/set-loan-allocation'
GET_SUB_ACCOUNT_BORROW_INTEREST_AND_LIMIT = '/api/v5/account/subaccount/interest-limits'

# Convert
GET_CURRENCIES = '/api/v5/asset/convert/currencies'
GET_CURRENCY_PAIR = '/api/v5/asset/convert/currency-pair'
ESTIMATE_QUOTE = '/api/v5/asset/convert/estimate-quote'
CONVERT_TRADE = '/api/v5/asset/convert/trade'
CONVERT_HISTORY = '/api/v5/asset/convert/history'

# FD Broker
FD_GET_REBATE_PER_ORDERS = '/api/v5/broker/fd/rebate-per-orders'
FD_REBATE_PER_ORDERS = '/api/v5/broker/fd/rebate-per-orders'

# BlockTrading
COUNTERPARTIES = '/api/v5/rfq/counterparties'
CREATE_RFQ = '/api/v5/rfq/create-rfq'
CANCEL_RFQ = '/api/v5/rfq/cancel-rfq'
CANCEL_BATCH_RFQS = '/api/v5/rfq/cancel-batch-rfqs'
CANCEL_ALL_RSQS = '/api/v5/rfq/cancel-all-rfqs'
EXECUTE_QUOTE = '/api/v5/rfq/execute-quote'
CREATE_QUOTE = '/api/v5/rfq/create-quote'
CANCEL_QUOTE = '/api/v5/rfq/cancel-quote'
CANCEL_BATCH_QUOTES = '/api/v5/rfq/cancel-batch-quotes'
CANCEL_ALL_QUOTES = '/api/v5/rfq/cancel-all-quotes'
GET_RFQS = '/api/v5/rfq/rfqs'
GET_QUOTES = '/api/v5/rfq/quotes'
GET_RFQ_TRADES = '/api/v5/rfq/trades'
GET_PUBLIC_TRADES = '/api/v5/rfq/public-trades'
MMP_RESET = '/api/v5/rfq/mmp-reset'
MARKER_INSTRUMENT_SETTING = '/api/v5/rfq/maker-instrument-settings'


# Trading Bot
GRID_ORDER_ALGO = '/api/v5/tradingBot/grid/order-algo'
GRID_AMEND_ORDER_ALGO = '/api/v5/tradingBot/grid/amend-order-algo'
GRID_STOP_ORDER_ALGO = '/api/v5/tradingBot/grid/stop-order-algo'
GRID_ORDERS_ALGO_PENDING = '/api/v5/tradingBot/grid/orders-algo-pending'
GRID_ORDERS_ALGO_HISTORY = '/api/v5/tradingBot/grid/orders-algo-history'
GRID_ORDERS_ALGO_DETAILS = '/api/v5/tradingBot/grid/orders-algo-details'
GRID_SUB_ORDERS = '/api/v5/tradingBot/grid/sub-orders'
GRID_POSITIONS = '/api/v5/tradingBot/grid/positions'
GRID_WITHDRAW_INCOME = '/api/v5/tradingBot/grid/withdraw-income'
GRID_COMPUTE_MARIGIN_BALANCE = '/api/v5/tradingBot/grid/compute-margin-balance'
GRID_MARGIN_BALANCE = '/api/v5/tradingBot/grid/margin-balance'
GRID_AI_PARAM = '/api/v5/tradingBot/grid/ai-param'
PLACE_RECURRING_BUY_ORDER = '/api/v5/tradingBot/recurring/order-algo'
AMEND_RECURRING_BUY_ORDER = '/api/v5/tradingBot/recurring/amend-order-algo'
STOP_RECURRING_BUY_ORDER = '/api/v5/tradingBot/recurring/stop-order-algo'
GET_RECURRING_BUY_ORDER_LIST = '/api/v5/tradingBot/recurring/orders-algo-pending'
GET_RECURRING_BUY_ORDER_HISTORY = '/api/v5/tradingBot/recurring/orders-algo-history'
GET_RECURRING_BUY_ORDER_DETAILS = '/api/v5/tradingBot/recurring/orders-algo-details'
GET_RECURRING_BUY_SUB_ORDERS = '/api/v5/tradingBot/recurring/sub-orders'

# Stacking
STACK_DEFI_OFFERS = '/api/v5/finance/staking-defi/offers'
STACK_DEFI_PURCHASE = '/api/v5/finance/staking-defi/purchase'
STACK_DEFI_REDEEM = '/api/v5/finance/staking-defi/redeem'
STACK_DEFI_CANCEL = '/api/v5/finance/staking-defi/cancel'
STACK_DEFI_ORDERS_ACTIVITY = '/api/v5/finance/staking-defi/orders-active'
STACK_DEFI_ORDERS_HISTORY = '/api/v5/finance/staking-defi/orders-history'
GET_SAVING_BALANCE = '/api/v5/finance/savings/balance'
SAVING_PURCHASE_REDEMPTION = '/api/v5/finance/savings/purchase-redempt'
SET_LENDING_RATE = '/api/v5/finance/savings/set-lending-rate'
GET_LENDING_HISTORY = '/api/v5/finance/savings/lending-history'
GET_PUBLIC_BORROW_INFO = '/api/v5/finance/savings/lending-rate-summary'
GET_PUBLIC_BORROW_HISTORY = '/api/v5/finance/savings/lending-rate-history'
STACK_ETH_PRODUCT_INFO = '/api/v5/finance/staking-defi/eth/product-info'
STACK_ETH_PURCHASE = '/api/v5/finance/staking-defi/eth/purchase'
STACK_ETH_REDEEM = '/api/v5/finance/staking-defi/eth/redeem'
STACK_ETH_BALANCE = '/api/v5/finance/staking-defi/eth/balance'
STACK_ETH_PURCHASE_REDEEM_HISTORY = '/api/v5/finance/staking-defi/eth/purchase-redeem-history'
STACK_ETH_APY_HISTORY = '/api/v5/finance/staking-defi/eth/apy-history'
STACK_SOL_PURCHASE = '/api/v5/finance/staking-defi/sol/purchase'
STACK_SOL_REDEEM = '/api/v5/finance/staking-defi/sol/redeem'
STACK_SOL_BALANCE = '/api/v5/finance/staking-defi/sol/balance'
STACK_SOL_PURCHASE_REDEEM_HISTORY = '/api/v5/finance/staking-defi/sol/purchase-redeem-history'
STACK_SOL_APY_HISTORY = '/api/v5/finance/staking-defi/sol/apy-history'
STACK_SOL_PRODUCT_INFO = '/api/v5/finance/staking-defi/sol/product-info'

# Status
STATUS = '/api/v5/system/status'

# Copy Trading
GET_EXISTING_LEADING_POSITIONS = '/api/v5/copytrading/current-subpositions'
GET_LEADING_POSITIONS_HISTORY = '/api/v5/copytrading/subpositions-history'
PLACE_LEADING_STOP_ORDER = '/api/v5/copytrading/algo-order'
CLOSE_LEADING_POSITIONS = '/api/v5/copytrading/close-subposition'
GET_LEADING_POSITIONS = '/api/v5/copytrading/instruments'
AMEND_EXISTING_LEADING_POSITIONS = '/api/v5/copytrading/set-instruments'
GET_PROFIT_SHARING_DETAILS = '/api/v5/copytrading/profit-sharing-details'
GET_TOTAL_PROFIT_SHARING = '/api/v5/copytrading/total-profit-sharing'
GET_UNREALIZED_PROFIT_SHARING_DETAILS = '/api/v5/copytrading/unrealized-profit-sharing-details'

# Spread Trading˚
SPREAD_PLACE_ORDER= '/api/v5/sprd/order'
SPREAD_CANCEL_ORDER = '/api/v5/sprd/cancel-order'
SPREAD_CANCEL_ALL_ORDERS = '/api/v5/sprd/mass-cancel'
SPREAD_GET_ORDER_DETAILS = '/api/v5/sprd/order'
SPREAD_GET_ACTIVE_ORDERS = '/api/v5/sprd/orders-pending'
SPREAD_GET_ORDERS = '/api/v5/sprd/orders-history'
SPREAD_GET_TRADES = '/api/v5/sprd/trades'
SPREAD_GET_SPREADS = '/api/v5/sprd/spreads'
SPREAD_GET_ORDER_BOOK = '/api/v5/sprd/books'
SPREAD_GET_TICKER = '/api/v5/sprd/ticker'
SPREAD_GET_PUBLIC_TRADES = '/api/v5/sprd/public-trades'

# Flexible loan
FINANCE_BORROW_CURRENCIES = '/api/v5/finance/flexible-loan/borrow-currencies'
FINANCE_COLLATERAL_ASSETS = '/api/v5/finance/flexible-loan/collateral-assets'
FINANCE_MAX_LOAN = '/api/v5/finance/flexible-loan/max-loan'
FINANCE_MAX_REDEEM = '/api/v5/finance/flexible-loan/max-collateral-redeem-amount'
FINANCE_ADJUST_COLLATERAL = '/api/v5/finance/flexible-loan/adjust-collateral'
FINANCE_LOAN_INFO = '/api/v5/finance/flexible-loan/loan-info'
FINANCE_LOAN_HISTORY = '/api/v5/finance/flexible-loan/loan-history'
FINANCE_INTEREST_ACCRUED = '/api/v5/finance/flexible-loan/interest-accrued'





from .restapi import *
from .wsapi import *
from .clients import OkxRestClient, OkxSocketClient

__all__ = [
    "OkxRestClient",
    "OkxSocketClient",
    "AccountClient",
    "AlgoTradingClient",
    "BlockTradingClient",
    "NonDisclosedBrokerClient",
    "FullyDisclosedBrokerClient",
    "CopyTradingClient",
    "FinanceClient",
    "FundingClient",
    "GridTradingClient",
    "PublicDataClient",
    "RecurringBuyClient",
    "RubikClient",
    "SignalTradingClient",
    "SpreadTradingClient",
    "SubAccountClient",
    "TradingClient",
]



from okx.okxclient import OkxClient
from okx.consts import *


class StakingDefiAPI(OkxClient):
    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1', domain = 'https://www.okx.com',debug = False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    def get_offers(self, productId='', protocolType='', ccy=''):
        params = {
            'productId': productId,
            'protocolType': protocolType,
            'ccy': ccy
        }
        return self._request_with_params(GET, STACK_DEFI_OFFERS, params)

    def purchase(self, productId='', investData=[], term='', tag=''):

        params = {
            'productId': productId,
            'investData': investData
        }
        if term != '':
            params['term'] = term
        if tag != '':
            params['tag'] = tag
        return self._request_with_params(POST, STACK_DEFI_PURCHASE, params)

    def redeem(self, ordId='', protocolType='', allowEarlyRedeem=''):
        params = {
            'ordId': ordId,
            'protocolType': protocolType,
            'allowEarlyRedeem': allowEarlyRedeem
        }
        return self._request_with_params(POST, STACK_DEFI_REDEEM, params)

    def cancel(self, ordId='', protocolType=''):
        params = {
            'ordId': ordId,
            'protocolType': protocolType
        }
        return self._request_with_params(POST, STACK_DEFI_CANCEL, params)

    def get_activity_orders(self, productId='', protocolType='', ccy='', state=''):
        params = {
            'productId': productId,
            'protocolType': protocolType,
            'ccy': ccy,
            'state': state
        }
        return self._request_with_params(GET, STACK_DEFI_ORDERS_ACTIVITY, params)

    def get_orders_history(self, productId='', protocolType='', ccy='', after='', before='', limit=''):
        params = {
            'productId': productId,
            'protocolType': protocolType,
            'ccy': ccy,
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, STACK_DEFI_ORDERS_HISTORY, params)




import asyncio
import logging
import ssl

import certifi
import websockets

logger = logging.getLogger(__name__)


class WebSocketFactory:

    def __init__(self, url):
        self.url = url
        self.websocket = None
        self.loop = asyncio.get_event_loop()

    async def connect(self):
        ssl_context = ssl.create_default_context()
        ssl_context.load_verify_locations(certifi.where())
        try:
            self.websocket = await websockets.connect(self.url, ssl=ssl_context)
            logger.info("WebSocket connection established.")
            return self.websocket
        except Exception as e:
            logger.error(f"Error connecting to WebSocket: {e}")
            return None

    async def close(self):
        if self.websocket:
            await self.websocket.close()
            self.websocket = None



from .BaseClient import OkxBaseClient
from ..constants import *


class PublicDataClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Get System Time
    def get_system_time(self):
        return self._request(GET, SYSTEM_TIME)

    # Status
    def status(self, state=""):
        params = {"state": state}
        return self._request(GET, SYSTEM_STATUS, params)

    # Get Instruments
    def get_instruments(self, instType, uly="", instId="", instFamily=""):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_INSTRUMENTS, params)

    # Get Delivery/Exercise History
    def get_delivery_exercise_history(
        self, instType, uly="", after="", before="", limit="", instFamily=""
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_DELIVERY_EXERCISE_HISTORY, params)

    # Get Open Interest
    def get_open_interest(self, instType, uly="", instId="", instFamily=""):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_OPEN_INTEREST, params)

    # Get Funding Rate
    def get_funding_rate(self, instId):
        params = {"instId": instId}
        return self._request(GET, PUBLIC_FUNDING_RATE, params)

    # Get Funding Rate History
    def funding_rate_history(self, instId, after="", before="", limit=""):
        params = {"instId": instId, "after": after, "before": before, "limit": limit}
        return self._request(GET, PUBLIC_FUNDING_RATE_HISTORY, params)

    # Get Limit Price
    def get_price_limit(self, instId):
        params = {"instId": instId}
        return self._request(GET, PUBLIC_PRICE_LIMIT, params)

    # Get Option Market Data
    def get_opt_summary(self, uly="", expTime="", instFamily=""):
        params = {"uly": uly, "expTime": expTime, "instFamily": instFamily}
        return self._request(GET, PUBLIC_OPT_SUMMARY, params)

    # Get Estimated Delivery/Excercise Price
    def get_estimated_price(self, instId):
        params = {"instId": instId}
        return self._request(GET, PUBLIC_ESTIMATED_PRICE, params)

    # Get Discount Rate And Interest-Free Quota
    def discount_interest_free_quota(self, ccy=""):
        params = {"ccy": ccy}
        return self._request(GET, PUBLIC_DISCOUNT_RATE_INTETEST_FREE_QUOTA, params)

    # Get Mark Price
    def get_mark_price(self, instType, uly="", instId="", instFamily=""):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_MARK_PRICE, params)

    # Get position tiers
    def get_position_tiers(
        self, instType, tdMode, uly="", instFamily="", instId="", ccy="", tier=""
    ):
        params = {
            "instType": instType,
            "tdMode": tdMode,
            "uly": uly,
            "instId": instId,
            "ccy": ccy,
            "tier": tier,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_POSITION_TIERS, params)

    # Get interest rate and loan quota
    def get_interest_rate_loan_quota(self):
        return self._request(GET, PUBLIC_INTEREST_RATE_LOAN_QUOTA)

    # Get interest rate and loan quota for VIP loans
    def get_vip_interest_rate_loan_quota(self):
        return self._request(GET, PUBLIC_VIP_INTEREST_RATE_LOAN_QUOTA)

    # Get underlying
    def get_underlying(self, instType=""):
        params = {"instType": instType}
        return self._request(GET, PUBLIC_UNDERLYING, params)

    # Get insurance fund
    def get_insurance_fund(
        self,
        instType="",
        type="",
        uly="",
        ccy="",
        before="",
        after="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "type": type,
            "uly": uly,
            "ccy": ccy,
            "before": before,
            "after": after,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, PUBLIC_INSURANCE_FUND, params)

    # Unit convert
    def unit_convert(self, type="", instId="", sz="", px="", unit=""):
        params = {"type": type, "instId": instId, "sz": sz, "px": px, "unit": unit}
        return self._request(GET, PUBLIC_CONVERT_CONTRACT_COIN, params)

    # Get option tick bands
    def get_option_tickBands(self, instType="", instFamily=""):
        params = {"instType": instType, "instFamily": instFamily}
        return self._request(GET, PUBLIC_INSTRUMENT_TICK_BANDS, params)

    # Get Index Tickers
    def get_index_tickers(self, quoteCcy="", instId=""):
        params = {"quoteCcy": quoteCcy, "instId": instId}
        return self._request(GET, MARKET_INDEX_TICKERS, params)

    # Get Index Candlesticks
    def get_index_candlesticks(self, instId, after="", before="", bar="", limit=""):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_INDEX_CANDLES, params)

    # Get Index Candlesticks History
    def get_index_candlesticks_history(
        self, instId, after="", before="", bar="", limit=""
    ):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_HISTORY_INDEX_CANDLES, params)

    # Get Mark Price Candlesticks
    def get_mark_price_candlesticks(
        self, instId, after="", before="", bar="", limit=""
    ):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_MARK_PRICE_CANDLES, params)

    # Get Mark Price Candlesticks History
    def get_mark_price_candlesticks_history(
        self, instId, after="", before="", bar="", limit=""
    ):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_HISTORY_MARK_PRICE_CANDLES, params)

    # Get Oracle
    def get_oracle(self):
        return self._request(GET, MARKET_OPEN_ORACLE)

    # Get exchange rate
    def get_exchange_rate(self):
        return self._request(GET, MARKET_EXCHANGE_RATE)

    # Get index components
    def get_index_components(self, index=""):
        param = {"index": index}
        return self._request(GET, MARKET_INDEX_COMPONENTS, param)

    # Get block tickers
    def get_block_tickers(self, instType="", uly="", instFamily=""):
        params = {"instType": instType, "uly": uly, "instFamily": instFamily}
        return self._request(GET, MARKET_BLOCK_TICKERS, params)

    # Get block ticker
    def get_block_ticker(self, instId=""):
        params = {"instId": instId}
        return self._request(GET, MARKET_BLOCK_TICKER, params)

    # Get block trades
    def get_block_trades(self, instId=""):
        params = {"instId": instId}
        return self._request(GET, MARKET_BLOCK_TRADES, params)

    # Get economic calendar data
    def get_economic_calendar(
        self, region="", importance="", before="", after="", limit=""
    ):
        params = {
            "region": region,
            "importance": importance,
            "before": before,
            "after": after,
            "limit": limit,
        }
        return self._request(GET, PUBLIC_ECONOMIC_CALENDAR, params)

    # Get Tickers
    def get_tickers(self, instType, uly="", instFamily=""):
        if uly:
            params = {"instType": instType, "uly": uly, "instFamily": instFamily}
        else:
            params = {"instType": instType, "instFamily": instFamily}
        return self._request(GET, MARKET_TICKERS, params)

    # Get Ticker
    def get_ticker(self, instId):
        params = {"instId": instId}
        return self._request(GET, MARKET_TICKER, params)

    # Get Order Book
    def get_orderbook(self, instId, sz=""):
        params = {"instId": instId, "sz": sz}
        return self._request(GET, MARKET_BOOKS, params)

    # Get Full Order Book
    def get_full_orderbook(self, instId, sz=""):
        params = {"instId": instId, "sz": sz}
        return self._request(GET, MARKET_BOOKS_FULL, params)

    # Get Candlesticks
    def get_candlesticks(self, instId, after="", before="", bar="", limit=""):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_CANDLES, params)

    # Get Candlesticks history
    def get_history_candlesticks(self, instId, after="", before="", bar="", limit=""):
        params = {
            "instId": instId,
            "after": after,
            "before": before,
            "bar": bar,
            "limit": limit,
        }
        return self._request(GET, MARKET_HISTORY_CANDLES, params)

    # Get Trades
    def get_trades(self, instId, limit=""):
        params = {"instId": instId, "limit": limit}
        return self._request(GET, MARKET_TRADES, params)

    # GET Trades history
    def get_history_trades(self, instId="", type="", after="", before="", limit=""):
        params = {
            "instId": instId,
            "type": type,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, MARKET_HISTORY_TRADES, params)

    # Option trades by instrument family
    def get_option_trades_by_family(self, instFamily=""):
        params = {"instFamily": instFamily}
        return self._request(GET, MARKET_OPTION_INSTRUMENT_FAMILY_TRADES, params)

    # Get option trades
    def get_option_trades(self, instId="", instFamily="", optType=""):
        params = {"instId": instId, "instFamily": instFamily, "optType": optType}
        return self._request(GET, PUBLIC_OPTION_TRADES, params)

    # Get 24H total volume
    def get_volume(self):
        return self._request(GET, MARKET_PLATFORM_24_VOLUME)



# Url
API_URL = "https://www.okx.com"
WS_URL_PUBLIC = "wss://ws.okx.com:8443/ws/v5/public"
WS_URL_PRIVATE = "wss://ws.okx.com:8443/ws/v5/private"
WS_URL_BUSINESS = "wss://ws.okx.com:8443/ws/v5/business"

# Methods
GET = "GET"
POST = "POST"

# Headers
CONTENT_TYPE = "Content-Type"
APPLICATION_JSON = "application/json"

# Headers
OK_ACCESS_KEY = "OK-ACCESS-KEY"
OK_ACCESS_SIGN = "OK-ACCESS-SIGN"
OK_ACCESS_TIMESTAMP = "OK-ACCESS-TIMESTAMP"
OK_ACCESS_PASSPHRASE = "OK-ACCESS-PASSPHRASE"

# Broker Id
BROKER_ID = None

# Trading Account Endpoints
ACCOUNT_BALANCE = "/api/v5/account/balance"
ACCOUNT_POSITIONS = "/api/v5/account/positions"
ACCOUNT_POSITIONS_HISTORY = "/api/v5/account/positions-history"
ACCOUNT_ACCOUNT_POSITION_RISK = "/api/v5/account/account-position-risk"
ACCOUNT_BILLS = "/api/v5/account/bills"
ACCOUNT_BILLS_ARCHIVE = "/api/v5/account/bills-archive"
ACCOUNT_CONFIG = "/api/v5/account/config"
ACCOUNT_SET_POSITION_MODE = "/api/v5/account/set-position-mode"
ACCOUNT_SET_LEVERAGE = "/api/v5/account/set-leverage"
ACCOUNT_MAX_SIZE = "/api/v5/account/max-size"
ACCOUNT_MAX_AVAIL_SIZE = "/api/v5/account/max-avail-size"
ACCOUNT_POSITION_MARGIN_BALANCE = "/api/v5/account/position/margin-balance"
ACCOUNT_LEVERAGE_INFO = "/api/v5/account/leverage-info"
ACCOUNT_ADJUST_LEVERAGE_INFO = "/api/v5/account/adjust-leverage-info"
ACCOUNT_MAX_LOAN = "/api/v5/account/max-loan"
ACCOUNT_TRADE_FEE = "/api/v5/account/trade-fee"
ACCOUNT_INTEREST_ACCRUED = "/api/v5/account/interest-accrued"
ACCOUNT_INTEREST_RATE = "/api/v5/account/interest-rate"
ACCOUNT_SET_GREEKS = "/api/v5/account/set-greeks"
ACCOUNT_ISOLATED_MODE = "/api/v5/account/set-isolated-mode"
ACCOUNT_MAX_WITHDRAWAL = "/api/v5/account/max-withdrawal"
ACCOUNT_RISK_STATE = "/api/v5/account/risk-state"
ACCOUNT_QUICK_MARGIN_BORROW_REPAY = "/api/v5/account/quick-margin-borrow-repay"
ACCOUNT_QUICK_MARGIN_BORROW_REPAY_HISTORY = (
    "/api/v5/account/quick-margin-borrow-repay-history"
)
ACCOUNT_BORROW_REPAY = "/api/v5/account/borrow-repay"
ACCOUNT_BORROW_REPAY_HISTORY = "/api/v5/account/borrow-repay-history"
ACCOUNT_VIP_INTEREST_ACCRUED = "/api/v5/account/vip-interest-accrued"
ACCOUNT_VIP_INTEREST_DEDUCTED = "/api/v5/account/vip-interest-deducted"
ACCOUNT_VIP_LOAN_ORDER_LIST = "/api/v5/account/vip-loan-order-list"
ACCOUNT_VIP_LOAN_ORDER_DETAIL = "/api/v5/account/vip-loan-order-detail"
ACCOUNT_INTEREST_LIMITS = "/api/v5/account/interest-limits"
ACCOUNT_SIMULATED_MARGIN = "/api/v5/account/simulated_margin"
ACCOUNT_POSITION_BUILDER = "/api/v5/account/position-builder"
ACCOUNT_GREEKS = "/api/v5/account/greeks"
ACCOUNT_POSITION_TIERS = "/api/v5/account/position-tiers"
ACCOUNT_SET_RISK_OFFSET_TYPE = "/api/v5/account/set-riskOffset-type"
ACCOUNT_ACTIVATE_OPTION = "/api/v5/account/activate-option"
ACCOUNT_SET_AUTO_LOAN = "/api/v5/account/set-auto-loan"
ACCOUNT_SET_ACCOUNT_LEVEL = "/api/v5/account/set-account-level"
ACCOUNT_MMP_RESET = "/api/v5/account/mmp-reset"
ACCOUNT_SET_MMP_CONFIG = "/api/v5/account/mmp-config"
ACCOUNT_GET_MMP_CONFIG = "/api/v5/account/mmp-config"

# Affiliate Endpoints
AFFILIATE_INVITEE_DETAIL = "/api/v5/affiliate/invitee/detail"
USERS_PARTNER_IF_REBATE = "/api/v5/users/partner/if-rebate"

# Order Book Trading >> Trade Endpoints
TRADE_PLACE_ORDER = "/api/v5/trade/order"
TRADE_BATCH_ORDERS = "/api/v5/trade/batch-orders"
TRADE_CANCEL_ORDER = "/api/v5/trade/cancel-order"
TRADE_CANCEL_BATCH_ORDERS = "/api/v5/trade/cancel-batch-orders"
TRADE_AMEND_ORDER = "/api/v5/trade/amend-order"
TRADE_AMEND_BATCH_ORDER = "/api/v5/trade/amend-batch-orders"
TRADE_CLOSE_POSITION = "/api/v5/trade/close-position"
TRADE_GET_ORDER = "/api/v5/trade/order"
TRADE_ORDERS_PENDING = "/api/v5/trade/orders-pending"
TRADE_ORDERS_HISTORY = "/api/v5/trade/orders-history"
TRADE_ORDERS_HISTORY_ARCHIVE = "/api/v5/trade/orders-history-archive"
TRADE_FILLS = "/api/v5/trade/fills"
TRADE_FILLS_HISTORY = "/api/v5/trade/fills-history"
TRADE_APPLY_ORDERS_FILLS_ARCHIVE = "/api/v5/trade/fills-archive"
TRADE_RETRIEVE_ORDERS_FILLS_ARCHIVE = "/api/v5/trade/fills-archive"
TRADE_EASY_CONVERT_CURRENCY_LIST = "/api/v5/trade/easy-convert-currency-list"
TRADE_EASY_CONVERT = "/api/v5/trade/easy-convert"
TRADE_CONVERT_EASY_HISTORY = "/api/v5/trade/easy-convert-history"
TRADE_ONE_CLICK_REPAY_CURRENCY_LIST = "/api/v5/trade/one-click-repay-currency-list"
TRADE_ONE_CLICK_REPAY = "/api/v5/trade/one-click-repay"
TRADE_ONE_CLICK_REPAY_HISTORY = "/api/v5/trade/one-click-repay-history"
TRADE_MASS_CANCEL = "/api/v5/trade/mass-cancel"
TRADE_CANCEL_ALL_AFTER = "/api/v5/trade/cancel-all-after"
TRADE_ACCOUNT_RATE_LIMIT = "/api/v5/trade/account-rate-limit"

# Order Book Trading >> Algo Trading Endpoints
TRADE_PLACE_ALGO_ORDER = "/api/v5/trade/order-algo"
TRADE_CANCEL_ALGOS = "/api/v5/trade/cancel-algos"
TRADE_AMEND_ALGOS = "/api/v5/trade/amend-algos"
TRADE_CANCEL_ADVANCE_ALGOS = "/api/v5/trade/cancel-advance-algos"
TRADE_GET_ALGO_ORDER = "/api/v5/trade/order-algo"
TRADE_ORDERS_ALGO_PENDING = "/api/v5/trade/orders-algo-pending"
TRADE_ORDERS_ALGO_HISTORY = "/api/v5/trade/orders-algo-history"

# Order Book Trading >> Grid Trading Endpoints
GRID_ORDER_ALGO = "/api/v5/tradingBot/grid/order-algo"
GRID_AMEND_ORDER_ALGO_BASIC = "/api/v5/tradingBot/grid/amend-order-algo-basic-param"
GRID_AMEND_ORDER_ALGO = "/api/v5/tradingBot/grid/amend-order-algo"
GRID_STOP_ORDER_ALGO = "/api/v5/tradingBot/grid/stop-order-algo"
GRID_CLOSE_POSITION = "/api/v5/tradingBot/grid/close-position"
GRID_CANCEL_CLOSE_ORDER = "/api/v5/tradingBot/grid/cancel-close-order"
GRID_ORDERS_ALGO_PENDING = "/api/v5/tradingBot/grid/orders-algo-pending"
GRID_ORDERS_ALGO_HISTORY = "/api/v5/tradingBot/grid/orders-algo-history"
GRID_ORDERS_ALGO_DETAILS = "/api/v5/tradingBot/grid/orders-algo-details"
GRID_SUB_ORDERS = "/api/v5/tradingBot/grid/sub-orders"
GRID_POSITIONS = "/api/v5/tradingBot/grid/positions"
GRID_WITHDRAW_INCOME = "/api/v5/tradingBot/grid/withdraw-income"
GRID_COMPUTE_MARGIN_BALANCE = "/api/v5/tradingBot/grid/compute-margin-balance"
GRID_MARGIN_BALANCE = "/api/v5/tradingBot/grid/margin-balance"
GRID_AI_PARAM = "/api/v5/tradingBot/grid/ai-param"
GRID_MIN_INVESTMENT = "/api/v5/tradingBot/grid/min-investment"
GRID_RSI_BACK_TESTING = "/api/v5/tradingBot/public/rsi-back-testing"

# Order Book Trading >> Signal Trading Endpoints
SIGNAL_CREATE_SIGNAL = "/api/v5/tradingBot/signal/create-signal"
SIGNAL_SIGNALS = "/api/v5/tradingBot/signal/signals"
SIGNAL_ORDER_ALGO = "/api/v5/tradingBot/signal/order-algo"
SIGNAL_STOP_ORDER_ALGO = "/api/v5/tradingBot/signal/stop-order-algo"
SIGNAL_MARGIN_BALANCE = "/api/v5/tradingBot/signal/margin-balance"
SIGNAL_AMEND_TPSL = "/api/v5/tradingBot/signal/amendTPSL"
SIGNAL_SET_INSTRUMENTS = "/api/v5/tradingBot/signal/set-instruments"
SIGNAL_ORDERS_ALGO_DETAILS_01 = "/api/v5/tradingBot/signal/orders-algo-details"
SIGNAL_ORDERS_ALGO_DETAILS_02 = "/api/v5/tradingBot/signal/orders-algo-details"
SIGNAL_ORDERS_ALGO_HISTORY = "/api/v5/tradingBot/signal/orders-algo-history"
SIGNAL_POSITIONS = "/api/v5/tradingBot/signal/positions"
SIGNAL_POSITIONS_HISTORY = "/api/v5/tradingBot/signal/positions-history"
SIGNAL_CLOSE_POSITION = "/api/v5/tradingBot/signal/close-position"
SIGNAL_SUB_ORDER = "/api/v5/tradingBot/signal/sub-order"
SIGNAL_CANCEL_SUB_ORDER = "/api/v5/tradingBot/signal/cancel-sub-order"
SIGNAL_SUB_ORDERS = "/api/v5/tradingBot/signal/sub-orders"
SIGNAL_EVENT_HISTORY = "/api/v5/tradingBot/signal/event-history"

# Order Book Trading >> Recurring Buy Endpoints
RECURRING_ORDER_ALGO = "/api/v5/tradingBot/recurring/order-algo"
RECURRING_AMEND_ORDER_ALGO = "/api/v5/tradingBot/recurring/amend-order-algo"
RECURRING_STOP_ORDER_ALGO = "/api/v5/tradingBot/recurring/stop-order-algo"
RECURRING_ORDERS_ALGO_PENDING = "/api/v5/tradingBot/recurring/orders-algo-pending"
RECURRING_ORDERS_ALGO_HISTORY = "/api/v5/tradingBot/recurring/orders-algo-history"
RECURRING_ORDERS_ALGO_DETAILS = "/api/v5/tradingBot/recurring/orders-algo-details"
RECURRING_SUB_ORDERS = "/api/v5/tradingBot/recurring/sub-orders"

# Order Book Trading >> Copy Trading Endpoints
COPYTRADING_CURRENT_SUBPOSITIONS = "/api/v5/copytrading/current-subpositions"
COPYTRADING_SUBPOSITIONS_HISTORY = "/api/v5/copytrading/subpositions-history"
COPYTRADING_ALGO_ORDER = "/api/v5/copytrading/algo-order"
COPYTRADING_CLOSE_SUBPOSITION = "/api/v5/copytrading/close-subposition"
COPYTRADING_INSTRUMENTS = "/api/v5/copytrading/instruments"
COPYTRADING_SET_INSTRUMENTS = "/api/v5/copytrading/set-instruments"
COPYTRADING_PROFIT_SHARING_DETAILS = "/api/v5/copytrading/profit-sharing-details"
COPYTRADING_TOTAL_PROFIT_SHARING = "/api/v5/copytrading/total-profit-sharing"
COPYTRADING_UNREALIZED_PROFIT_SHARING_DETAILS = (
    "/api/v5/copytrading/unrealized-profit-sharing-details"
)
COPYTRADING_TOTAL_UNREALIZED_PROFIT_SHARING = (
    "/api/v5/copytrading/total-unrealized-profit-sharing"
)
COPYTRADING_APPLY_LEAD_TRADING = "/api/v5/copytrading/apply-lead-trading"
COPYTRADING_STOP_LEAD_TRADING = "/api/v5/copytrading/stop-lead-trading"
COPYTRADING_AMEND_PROFIT_SHARING_RATIO = (
    "/api/v5/copytrading/amend-profit-sharing-ratio"
)
COPYTRADING_CONFIG = "/api/v5/copytrading/config"
COPYTRADING_FIRST_COPY_SETTINGS = "/api/v5/copytrading/first-copy-settings"
COPYTRADING_AMEND_COPY_SETTINGS = "/api/v5/copytrading/amend-copy-settings"
COPYTRADING_STOP_COPY_TRADING = "/api/v5/copytrading/stop-copy-trading"
COPYTRADING_COPY_SETTINGS = "/api/v5/copytrading/copy-settings"
COPYTRADING_BATCH_LEVERAGE_INFO = "/api/v5/copytrading/batch-leverage-info"
COPYTRADING_BATCH_SET_LEVERAGE = "/api/v5/copytrading/batch-set-leverage"
COPYTRADING_CURRENT_LEAD_TRADERS = "/api/v5/copytrading/current-lead-traders"
COPYTRADING_LEAD_TRADERS_HISTORY = "/api/v5/copytrading/lead-traders-history"
COPYTRADING_PUBLIC_CONFIG = "/api/v5/copytrading/public-config"
COPYTRADING_PUBLIC_LEAD_TRADERS = "/api/v5/copytrading/public-lead-traders"
COPYTRADING_PUBLIC_WEEKLY_PNL = "/api/v5/copytrading/public-weekly-pnl"
COPYTRADING_PUBLIC_PNL = "/api/v5/copytrading/public-pnl"
COPYTRADING_PUBLIC_STATS = "/api/v5/copytrading/public-stats"
COPYTRADING_PUBLIC_PREFERENCE_CURRENCY = (
    "/api/v5/copytrading/public-preference-currency"
)
COPYTRADING_PUBLIC_CURRENT_SUBPOSITIONS = (
    "/api/v5/copytrading/public-current-subpositions"
)
COPYTRADING_PUBLIC_SUBPOSITIONS_HISTORY = (
    "/api/v5/copytrading/public-subpositions-history"
)
COPYTRADING_PUBLIC_COPY_TRADERS = "/api/v5/copytrading/public-copy-traders"
COPYTRADING_LEAD_TRADERS = "/api/v5/copytrading/lead-traders"
COPYTRADING_WEEKLY_PNL = "/api/v5/copytrading/weekly-pnl"
COPYTRADING_PNL = "/api/v5/copytrading/pnl"
COPYTRADING_STATS = "/api/v5/copytrading/stats"
COPYTRADING_PREFERENCE_CURRENCY = "/api/v5/copytrading/preference-currency"
COPYTRADING_PRIVATE_CURRENT_SUBPOSITIONS = (
    "/api/v5/copytrading/private-current-subpositions"
)
COPYTRADING_PERFORMANCE_SUBPOSITIONS_HISTORY = (
    "/api/v5/copytrading/performance-subpositions-history"
)
COPYTRADING_COPY_TRADERS = "/api/v5/copytrading/copy-traders"

# System Endpoints
SYSTEM_TIME = "/api/v5/public/time"
SYSTEM_STATUS = "/api/v5/system/status"

# Public Data Endpoints
PUBLIC_INSTRUMENTS = "/api/v5/public/instruments"
PUBLIC_DELIVERY_EXERCISE_HISTORY = "/api/v5/public/delivery-exercise-history"
PUBLIC_OPEN_INTEREST = "/api/v5/public/open-interest"
PUBLIC_FUNDING_RATE = "/api/v5/public/funding-rate"
PUBLIC_FUNDING_RATE_HISTORY = "/api/v5/public/funding-rate-history"
PUBLIC_PRICE_LIMIT = "/api/v5/public/price-limit"
PUBLIC_OPT_SUMMARY = "/api/v5/public/opt-summary"
PUBLIC_ESTIMATED_PRICE = "/api/v5/public/estimated-price"
PUBLIC_DISCOUNT_RATE_INTETEST_FREE_QUOTA = (
    "/api/v5/public/discount-rate-interest-free-quota"
)
PUBLIC_MARK_PRICE = "/api/v5/public/mark-price"
PUBLIC_POSITION_TIERS = "/api/v5/public/position-tiers"
PUBLIC_INTEREST_RATE_LOAN_QUOTA = "/api/v5/public/interest-rate-loan-quota"
PUBLIC_VIP_INTEREST_RATE_LOAN_QUOTA = "/api/v5/public/vip-interest-rate-loan-quota"
PUBLIC_UNDERLYING = "/api/v5/public/underlying"
PUBLIC_INSURANCE_FUND = "/api/v5/public/insurance-fund"
PUBLIC_CONVERT_CONTRACT_COIN = "/api/v5/public/convert-contract-coin"
PUBLIC_INSTRUMENT_TICK_BANDS = "/api/v5/public/instrument-tick-bands"
MARKET_INDEX_TICKERS = "/api/v5/market/index-tickers"
MARKET_INDEX_CANDLES = "/api/v5/market/index-candles"
MARKET_HISTORY_INDEX_CANDLES = "/api/v5/market/history-index-candles"
MARKET_MARK_PRICE_CANDLES = "/api/v5/market/mark-price-candles"
MARKET_HISTORY_MARK_PRICE_CANDLES = "/api/v5/market/history-mark-price-candles"
MARKET_OPEN_ORACLE = "/api/v5/market/open-oracle"
MARKET_EXCHANGE_RATE = "/api/v5/market/exchange-rate"
MARKET_INDEX_COMPONENTS = "/api/v5/market/index-components"
MARKET_BLOCK_TICKERS = "/api/v5/market/block-tickers"
MARKET_BLOCK_TICKER = "/api/v5/market/block-ticker"
MARKET_BLOCK_TRADES = "/api/v5/market/block-trades"
PUBLIC_ECONOMIC_CALENDAR = "/api/v5/public/economic-calendar"

# Market Data Endpoints
MARKET_TICKERS = "/api/v5/market/tickers"
MARKET_TICKER = "/api/v5/market/ticker"
MARKET_BOOKS = "/api/v5/market/books"
MARKET_BOOKS_FULL = "/api/v5/market/books-full"
MARKET_CANDLES = "/api/v5/market/candles"
MARKET_HISTORY_CANDLES = "/api/v5/market/history-candles"
MARKET_TRADES = "/api/v5/market/trades"
MARKET_HISTORY_TRADES = "/api/v5/market/history-trades"
MARKET_OPTION_INSTRUMENT_FAMILY_TRADES = (
    "/api/v5/market/option/instrument-family-trades"
)
PUBLIC_OPTION_TRADES = "/api/v5/public/option-trades"
MARKET_PLATFORM_24_VOLUME = "/api/v5/market/platform-24-volume"

# Block Trading Endpoints
RFQ_COUNTERPARTIES = "/api/v5/rfq/counterparties"
RFQ_CREATE_RFQ = "/api/v5/rfq/create-rfq"
RFQ_CANCEL_RFQ = "/api/v5/rfq/cancel-rfq"
RFQ_CANCEL_BATCH_RFQS = "/api/v5/rfq/cancel-batch-rfqs"
RFQ_CANCEL_ALL_RSQS = "/api/v5/rfq/cancel-all-rfqs"
RFQ_EXECUTE_QUOTE = "/api/v5/rfq/execute-quote"
RFQ_GET_MARKER_INSTRUMENT_SETTING = "/api/v5/rfq/maker-instrument-settings"
RFQ_SET_MARKER_INSTRUMENT_SETTING = "/api/v5/rfq/maker-instrument-settings"
RFQ_MMP_RESET = "/api/v5/rfq/mmp-reset"
RFQ_SET_MMP_CONFIG = "/api/v5/rfq/mmp-config"
RFQ_GET_MMP_CONFIG = "/api/v5/rfq/mmp-config"
RFQ_CREATE_QUOTE = "/api/v5/rfq/create-quote"
RFQ_CANCEL_QUOTE = "/api/v5/rfq/cancel-quote"
RFQ_CANCEL_BATCH_QUOTES = "/api/v5/rfq/cancel-batch-quotes"
RFQ_CANCEL_ALL_QUOTES = "/api/v5/rfq/cancel-all-quotes"
RFQ_RFQS = "/api/v5/rfq/rfqs"
RFQ_QUOTES = "/api/v5/rfq/quotes"
RFQ_TRADES = "/api/v5/rfq/trades"
RFQ_PUBLIC_TRADES = "/api/v5/rfq/public-trades"

# Spread Trading Endpoints
SPREAD_PLACE_ORDER = "/api/v5/sprd/order"
SPREAD_CANCEL_ORDER = "/api/v5/sprd/cancel-order"
SPREAD_MASS_CANCEL = "/api/v5/sprd/mass-cancel"
SPREAD_AMEND_ORDER = "/api/v5/sprd/amend-order"
SPREAD_GET_ORDER = "/api/v5/sprd/order"
SPREAD_ORDERS_PENDING = "/api/v5/sprd/orders-pending"
SPREAD_ORDERS_HISTORY = "/api/v5/sprd/orders-history"
SPREAD_ORDERS_HISTORY_ARCHIVE = "/api/v5/sprd/orders-history-archive"
SPREAD_TRADES = "/api/v5/sprd/trades"
SPREAD_SPREADS = "/api/v5/sprd/spreads"
SPREAD_BOOKS = "/api/v5/sprd/books"
SPREAD_TICKER = "/api/v5/sprd/ticker"
SPREAD_PUBLIC_TRADES = "/api/v5/sprd/public-trades"

# Trading Statistics Endpoints
RUBIK_STAT_TRADING_DATA_SUPPORT_COIN = "/api/v5/rubik/stat/trading-data/support-coin"
RUBIK_STAT_TAKER_VOLUME = "/api/v5/rubik/stat/taker-volume"
RUBIK_STAT_MARGIN_LOAN_RATIO = "/api/v5/rubik/stat/margin/loan-ratio"
RUBIK_STAT_CONTRACTS_LONG_SHORT_RATIO = (
    "/api/v5/rubik/stat/contracts/long-short-account-ratio"
)
RUBIK_STAT_CONTRACTS_INTEREST_VOLUME = (
    "/api/v5/rubik/stat/contracts/open-interest-volume"
)
RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME = (
    "/api/v5/rubik/stat/option/open-interest-volume"
)
RUBIK_STAT_OPTION_OPEN_INTEREST_RATIO = (
    "/api/v5/rubik/stat/option/open-interest-volume-ratio"
)
RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME_EXPIRY = (
    "/api/v5/rubik/stat/option/open-interest-volume-expiry"
)
RUBIK_STAT_OPTION_OPEN_INTEREST_VOLUME_STRIKE = (
    "/api/v5/rubik/stat/option/open-interest-volume-strike"
)
RUBIK_STAT_OPTION_TAKER_BLOCK_VOLUME = "/api/v5/rubik/stat/option/taker-block-volume"

# Funding Account Endpoints
ASSET_CURRENCIES = "/api/v5/asset/currencies"
ASSET_BALANCES = "/api/v5/asset/balances"
ASSET_NON_TRADABLE_ASSETS = "/api/v5/asset/non-tradable-assets"
ASSET_ASSET_VALUATION = "/api/v5/asset/asset-valuation"
ASSET_TRANSFER = "/api/v5/asset/transfer"
ASSET_TRANSFER_STATE = "/api/v5/asset/transfer-state"
ASSET_BILLS = "/api/v5/asset/bills"
ASSET_DEPOSIT_LIGHTNING = "/api/v5/asset/deposit-lightning"
ASSET_DEPOSIT_ADDRESS = "/api/v5/asset/deposit-address"
ASSET_DEPOSIT_HISTORY = "/api/v5/asset/deposit-history"
ASSET_WITHDRAWAL = "/api/v5/asset/withdrawal"
ASSET_WITHDRAWAL_LIGHTNING = "/api/v5/asset/withdrawal-lightning"
ASSET_CANCEL_WITHDRAWAL = "/api/v5/asset/cancel-withdrawal"
ASSET_WITHDRAWAL_HISTORY = "/api/v5/asset/withdrawal-history"
ASSET_DEPOSIT_WITHDRAW_STATUS = "/api/v5/asset/deposit-withdraw-status"
ASSET_CONVERT_DUST_ASSETS = "/api/v5/asset/convert-dust-assets"
ASSET_EXCHANGE_LIST = "/api/v5/asset/exchange-list"
ASSET_MONTHLY_STATEMENT_APPLY = "/api/v5/asset/monthly-statement"
ASSET_MONTHLY_STATEMENT_RETRIEVE = "/api/v5/asset/monthly-statement"
ASSET_CONVERT_CURRENCIES = "/api/v5/asset/convert/currencies"
ASSET_CONVERT_CURRENCY_PAIR = "/api/v5/asset/convert/currency-pair"
ASSET_CONVERT_ESTIMATE_QUOTE = "/api/v5/asset/convert/estimate-quote"
ASSET_CONVERT_TRADE = "/api/v5/asset/convert/trade"
ASSET_CONVERT_HISTORY = "/api/v5/asset/convert/history"

# SubAccount Endpoints
USERS_SUBACCOUNT_LIST = "/api/v5/users/subaccount/list"
USERS_SUBACCOUNT_MODIFY_APIKEY = "/api/v5/users/subaccount/modify-apikey"
ACCOUNT_SUBACCOUNT_BALANCES = "/api/v5/account/subaccount/balances"
ASSET_SUBACCOUNT_BALANCES = "/api/v5/asset/subaccount/balances"
ACCOUNT_SUBACCOUNT_MAX_WITHDRAWAL = "/api/v5/account/subaccount/max-withdrawal"
ASSET_SUBACCOUNT_BILLS = "/api/v5/asset/subaccount/bills"
ASSET_SUBACCOUNT_MANAGED_SUBACCOUNT_BILLS = (
    "/api/v5/asset/subaccount/managed-subaccount-bills"
)
ASSET_SUBACCOUNT_TRANSFER = "/api/v5/asset/subaccount/transfer"
USERS_SUBACCOUNT_SET_TRANSFER_OUT = "/api/v5/users/subaccount/set-transfer-out"
USERS_ENTRUST_SUBACCOUNT_LIST = "/api/v5/users/entrust-subaccount-list"
ACCOUNT_SUBACCOUNT_SET_LOAN_ALLOCATION = (
    "/api/v5/account/subaccount/set-loan-allocation"
)
ACCOUNT_SUBACCOUNT_INTEREST_LIMITS = "/api/v5/account/subaccount/interest-limits"

# Staking Endpoints
FINANCE_STACKING_DEFI_OFFERS = "/api/v5/finance/staking-defi/offers"
FINANCE_STACKING_DEFI_PURCHASE = "/api/v5/finance/staking-defi/purchase"
FINANCE_STACKING_DEFI_REDEEM = "/api/v5/finance/staking-defi/redeem"
FINANCE_STACKING_DEFI_CANCEL = "/api/v5/finance/staking-defi/cancel"
FINANCE_STACKING_DEFI_ORDERS_ACTIVE = "/api/v5/finance/staking-defi/orders-active"
FINANCE_STACKING_DEFI_ORDERS_HISTORY = "/api/v5/finance/staking-defi/orders-history"
FINANCE_STACKING_DEFI_ETH_PURCHASE = "/api/v5/finance/staking-defi/eth/purchase"
FINANCE_STACKING_DEFI_ETH_REDEEM = "/api/v5/finance/staking-defi/eth/redeem"
FINANCE_STACKING_DEFI_ETH_BALANCE = "/api/v5/finance/staking-defi/eth/balance"
FINANCE_STACKING_DEFI_ETH_PURCHASE_REDEEM_HISTORY = (
    "/api/v5/finance/staking-defi/eth/purchase-redeem-history"
)
FINANCE_STACKING_DEFI_ETH_APY_HISTORY = "/api/v5/finance/staking-defi/eth/apy-history"
FINANCE_SAVINGS_BALANCE = "/api/v5/finance/savings/balance"
FINANCE_SAVINGS_PURCHASE_REDEMPT = "/api/v5/finance/savings/purchase-redempt"
FINANCE_SAVINGS_SET_LENDING_RATE = "/api/v5/finance/savings/set-lending-rate"
FINANCE_SAVINGS_LENDING_HISTORY = "/api/v5/finance/savings/lending-history"
FINANCE_SAVINGS_LENDING_RATE_SUMMARY = "/api/v5/finance/savings/lending-rate-summary"
FINANCE_SAVINGS_LENDING_RATE_HISTORY = "/api/v5/finance/savings/lending-rate-history"

# Non-Disclosed Broker Endpoints
BROKER_ND_INFO = "/api/v5/broker/nd/info"
BROKER_ND_CREATE_SUBACCOUNT = "/api/v5/broker/nd/create-subaccount"
BROKER_ND_DELETE_SUBACCOUNT = "/api/v5/broker/nd/delete-subaccount"
BROKER_ND_SUBACCOUNT_INFO = "/api/v5/broker/nd/subaccount-info"
BROKER_ND_CREATE_APIKEY = "/api/v5/broker/nd/subaccount/apikey"
BROKER_ND_SELECT_APIKEY = "/api/v5/broker/nd/subaccount/apikey"
BROKER_ND_MODIFY_APIKEY = "/api/v5/broker/nd/subaccount/modify-apikey"
BROKER_ND_DELETE_APIKEY = "/api/v5/broker/nd/subaccount/delete-apikey"
BROKER_ND_SET_SUBACCOUNT_LEVEL = "/api/v5/broker/nd/set-subaccount-level"
BROKER_ND_SET_SUBACCOUNT_FEE_RATE = "/api/v5/broker/nd/set-subaccount-fee-rate"
BROKER_ND_SUBACCOUNT_DEPOSIT_ADDRESS = (
    "/api/v5/asset/broker/nd/subaccount-deposit-address"
)
BROKER_ND_MODIFY_SUBACCOUNT_DEPOSIT_ADDRESS = (
    "/api/v5/asset/broker/nd/modify-subaccount-deposit-address"
)
BROKER_ND_SUBACCOUNT_DEPOSIT_ADDRESS = (
    "/api/v5/asset/broker/nd/subaccount-deposit-address"
)
BROKER_ND_SUBACCOUNT_DEPOSIT_HISTORY = (
    "/api/v5/asset/broker/nd/subaccount-deposit-history"
)
BROKER_ND_SUBACCOUNT_WITHDRAWAL_HISTORY = (
    "/api/v5/asset/broker/nd/subaccount-withdrawal-history"
)
BROKER_ND_REBATE_DAILY = "/api/v5/broker/nd/rebate-daily"
BROKER_ND_GET_REBATE_PER_ORDERS = "/api/v5/broker/nd/rebate-per-orders"
BROKER_ND_GEN_REBATE_PER_ORDERS = "/api/v5/broker/nd/rebate-per-orders"
FINANCE_SFP_DCD_PRODUCTS = "/api/v5/finance/sfp/dcd/products"
FINANCE_SFP_DCD_QUOTE = "/api/v5/finance/sfp/dcd/quote"
FINANCE_SFP_DCD_ORDER = "/api/v5/finance/sfp/dcd/order"
FINANCE_SFP_DCD_ORDERS = "/api/v5/finance/sfp/dcd/orders"
BROKER_ND_SET_SUBACCOUNT_ASSETS = "/api/v5/broker/nd/set-subaccount-assets"
BROKER_ND_REPORT_SUBACCOUNT_IP = "/api/v5/broker/nd/report-subaccount-ip"
BROKER_ND_IF_REBATE = "/api/v5/broker/nd/if-rebate"

# Fully-Disclosed Broker Endpoints
BROKER_FD_GET_REBATE_PER_ORDERS = "/api/v5/broker/fd/rebate-per-orders"
BROKER_FD_GEN_REBATE_PER_ORDERS = "/api/v5/broker/fd/rebate-per-orders"
BROKER_FD_IF_REBATE = "/api/v5/broker/fd/if-rebate"



from .BaseClient import OkxBaseClient
from ..constants import *


class SignalTradingClient(OkxBaseClient):
    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Create signal
    def create_signal(self, signalChanName, signalChanDesc=""):
        params = {"signalChanName": signalChanName, "signalChanDesc": signalChanDesc}
        return self._request(POST, SIGNAL_CREATE_SIGNAL, params)

    # Get signals
    def get_signals(
        self, signalSourceType, signalChanId="", after="", before="", limit=""
    ):
        params = {
            "signalSourceType": signalSourceType,
            "signalChanId": signalChanId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, SIGNAL_SIGNALS, params)

    # Create signal bot
    def create(
        self,
        signalChanId,
        lever,
        investAmt,
        subOrdType,
        includeAll="",
        instIds="",
        ratio="",
        entrySettingParam={},
        exitSettingParam={},
    ):
        params = {
            "signalChanId": signalChanId,
            "lever": lever,
            "investAmt": investAmt,
            "subOrdType": subOrdType,
            "includeAll": includeAll,
            "instIds": instIds,
            "ratio": ratio,
            "entrySettingParam": entrySettingParam,
            "exitSettingParam": exitSettingParam,
        }
        return self._request(POST, SIGNAL_ORDER_ALGO, params)

    # Cancel signal bot
    def cancel(self, algoId):
        params = {"algoId": algoId}
        return self._request(POST, SIGNAL_STOP_ORDER_ALGO, params)

    # Adjust margin balance
    def adjust_margin_balance(self, algoId, type, amt, allowReinvest=False):
        params = {
            "algoId": algoId,
            "type": type,
            "amt": amt,
            "allowReinvest": allowReinvest,
        }
        return self._request(POST, SIGNAL_MARGIN_BALANCE, params)

    # Amend TP SL
    def amend_tpsl(self, algoId, exitSettingParam={}):
        params = {"algoId": algoId, "exitSettingParam": exitSettingParam}
        return self._request(POST, SIGNAL_AMEND_TPSL, params)

    # Set instruments
    def set_instruments(self, algoId, instIds=[], includeAll=False):
        params = {"algoId": algoId, "instIds": instIds, "includeAll": includeAll}
        return self._request(POST, SIGNAL_SET_INSTRUMENTS, params)

    # Get Signal bot order details
    def get_order(self, algoOrdType, algoId):
        params = {"algoOrdType": algoOrdType, "algoId": algoId}
        return self._request(GET, SIGNAL_ORDERS_ALGO_DETAILS_01, params)

    # Get Active signal bot
    def get_active(self, algoOrdType, algoId, after="", before="", limit=""):
        params = {
            "algoOrdType": algoOrdType,
            "algoId": algoId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, SIGNAL_ORDERS_ALGO_DETAILS_02, params)

    # Get Signal bot history
    def get_history(self, algoOrdType, algoId, after="", before="", limit=""):
        params = {
            "algoOrdType": algoOrdType,
            "algoId": algoId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, SIGNAL_ORDERS_ALGO_HISTORY, params)

    # Get Signal bot order positions
    def get_positions(self, algoOrdType, algoId):
        params = {"algoOrdType": algoOrdType, "algoId": algoId}
        return self._request(GET, SIGNAL_POSITIONS, params)

    # Get Signal bot order positions
    def get_position_history(self, algoId="", instId="", after="", before="", limit=""):
        params = {
            "algoId": algoId,
            "instId": instId,
            "after": after,
            "before": before,
            "limit": limit,
        }
        return self._request(GET, SIGNAL_POSITIONS_HISTORY, params)

    # Close position
    def close_position(self, algoId, instId):
        params = {"algoId": algoId, "instId": instId}
        return self._request(POST, SIGNAL_CLOSE_POSITION, params)

    # Place sub order
    def place_sub_order(
        self, algoId, instId, side, ordType, sz, px="", reduceOnly=False
    ):
        params = {
            "algoId": algoId,
            "instId": instId,
            "side": side,
            "ordType": ordType,
            "sz": sz,
            "px": px,
            "reduceOnly": reduceOnly,
        }
        return self._request(POST, SIGNAL_SUB_ORDER, params)

    # Cancel sub order
    def cancel_sub_order(self, algoId, instId, signalOrdId):
        params = {"algoId": algoId, "instId": instId, "signalOrdId": signalOrdId}
        return self._request(POST, SIGNAL_CANCEL_SUB_ORDER, params)

    # Get sub orders
    def get_sub_orders(
        self,
        algoId,
        algoOrdType,
        type="",
        clOrdId="",
        state="",
        signalOrdId="",
        after="",
        before="",
        begin="",
        end="",
        limit="",
    ):
        params = {
            "algoId": algoId,
            "algoOrdType": algoOrdType,
            "type": type,
            "clOrdId": clOrdId,
            "state": state,
            "signalOrdId": signalOrdId,
            "after": after,
            "before": before,
            "begin": begin,
            "end": end,
            "limit": limit,
        }
        return self._request(GET, SIGNAL_SUB_ORDERS, params)

    # Get bot events
    def get_bot_events(self, algoId, after="", before="", limit=""):
        params = {"algoId": algoId, "after": after, "before": before, "limit": limit}
        return self._request(GET, SIGNAL_EVENT_HISTORY, params)



from .BaseClient import OkxBaseClient
from ..constants import *


class TradingClient(OkxBaseClient):

    def __init__(
        self,
        apikey="",
        apisecret="",
        passphrase="",
        use_server_time=False,
        simulation=False,
        domain=API_URL,
        debug=False,
        proxy=None,
    ):
        OkxBaseClient.__init__(
            self,
            apikey,
            apisecret,
            passphrase,
            use_server_time,
            simulation,
            domain,
            debug,
            proxy,
        )

    # Place Order
    def place_order(
        self,
        instId,
        tdMode,
        side,
        ordType,
        sz,
        ccy="",
        clOrdId="",
        posSide="",
        px="",
        reduceOnly="",
        tgtCcy="",
        tpTriggerPx="",
        tpOrdPx="",
        slTriggerPx="",
        slOrdPx="",
        tpTriggerPxType="",
        slTriggerPxType="",
        quickMgnType="",
        stpId="",
        stpMode="",
        attachAlgoOrds=None,
    ):
        params = {
            "instId": instId,
            "tdMode": tdMode,
            "side": side,
            "ordType": ordType,
            "sz": sz,
            "ccy": ccy,
            "clOrdId": clOrdId,
            "tag": BROKER_ID,
            "posSide": posSide,
            "px": px,
            "reduceOnly": reduceOnly,
            "tgtCcy": tgtCcy,
            "tpTriggerPx": tpTriggerPx,
            "tpOrdPx": tpOrdPx,
            "slTriggerPx": slTriggerPx,
            "slOrdPx": slOrdPx,
            "tpTriggerPxType": tpTriggerPxType,
            "slTriggerPxType": slTriggerPxType,
            "quickMgnType": quickMgnType,
            "stpId": stpId,
            "stpMode": stpMode,
            "attachAlgoOrds": attachAlgoOrds,
        }
        return self._request(POST, TRADE_PLACE_ORDER, params)

    # Place Multiple Orders
    def place_multiple_orders(self, orders_data):
        return self._request(POST, TRADE_BATCH_ORDERS, orders_data)

    # Cancel Order
    def cancel_order(self, instId, ordId="", clOrdId=""):
        params = {"instId": instId, "ordId": ordId, "clOrdId": clOrdId}
        return self._request(POST, TRADE_CANCEL_ORDER, params)

    # Cancel Multiple Orders
    def cancel_multiple_orders(self, orders_data):
        return self._request(POST, TRADE_CANCEL_BATCH_ORDERS, orders_data)

    # Amend Order
    def amend_order(
        self,
        instId,
        cxlOnFail="",
        ordId="",
        clOrdId="",
        reqId="",
        newSz="",
        newPx="",
        newTpTriggerPx="",
        newTpOrdPx="",
        newSlTriggerPx="",
        newSlOrdPx="",
        newTpTriggerPxType="",
        newSlTriggerPxType="",
        attachAlgoOrds="",
    ):
        params = {
            "instId": instId,
            "cxlOnFailc": cxlOnFail,
            "ordId": ordId,
            "clOrdId": clOrdId,
            "reqId": reqId,
            "newSz": newSz,
            "newPx": newPx,
            "newTpTriggerPx": newTpTriggerPx,
            "newTpOrdPx": newTpOrdPx,
            "newSlTriggerPx": newSlTriggerPx,
            "newSlOrdPx": newSlOrdPx,
            "newTpTriggerPxType": newTpTriggerPxType,
            "newSlTriggerPxType": newSlTriggerPxType,
        }
        params["attachAlgoOrds"] = attachAlgoOrds
        return self._request(POST, TRADE_AMEND_ORDER, params)

    # Amend Multiple Orders
    def amend_multiple_orders(self, orders_data):
        return self._request(POST, TRADE_AMEND_BATCH_ORDER, orders_data)

    # Close Positions
    def close_positions(
        self, instId, mgnMode, posSide="", ccy="", autoCxl="", clOrdId=""
    ):
        params = {
            "instId": instId,
            "mgnMode": mgnMode,
            "posSide": posSide,
            "ccy": ccy,
            "autoCxl": autoCxl,
            "clOrdId": clOrdId,
            "tag": BROKER_ID,
        }
        return self._request(POST, TRADE_CLOSE_POSITION, params)

    # Get Order Details
    def get_order(self, instId, ordId="", clOrdId=""):
        params = {"instId": instId, "ordId": ordId, "clOrdId": clOrdId}
        return self._request(GET, TRADE_GET_ORDER, params)

    # Get Order List
    def get_order_list(
        self,
        instType="",
        uly="",
        instId="",
        ordType="",
        state="",
        after="",
        before="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "ordType": ordType,
            "state": state,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, TRADE_ORDERS_PENDING, params)

    # Get Order History (last 7 days）
    def get_orders_history(
        self,
        instType,
        uly="",
        instId="",
        ordType="",
        state="",
        after="",
        before="",
        begin="",
        end="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "ordType": ordType,
            "state": state,
            "after": after,
            "before": before,
            "begin": begin,
            "end": end,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, TRADE_ORDERS_HISTORY, params)

    # Get Order History (last 3 months)
    def get_orders_history_archive(
        self,
        instType,
        uly="",
        instId="",
        ordType="",
        state="",
        after="",
        before="",
        begin="",
        end="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "ordType": ordType,
            "state": state,
            "after": after,
            "before": before,
            "begin": begin,
            "end": end,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, TRADE_ORDERS_HISTORY_ARCHIVE, params)

    # Get Transaction Details (last 3 days）
    def get_fills(
        self,
        instType="",
        uly="",
        instId="",
        ordId="",
        after="",
        before="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, TRADE_FILLS, params)

    # Get Transaction Details (last 3 months)
    def get_fills_history(
        self,
        instType,
        uly="",
        instId="",
        ordId="",
        after="",
        before="",
        limit="",
        instFamily="",
    ):
        params = {
            "instType": instType,
            "uly": uly,
            "instId": instId,
            "ordId": ordId,
            "after": after,
            "before": before,
            "limit": limit,
            "instFamily": instFamily,
        }
        return self._request(GET, TRADE_FILLS_HISTORY, params)

    def apply_fills_archive(self, year, quarter):
        params = {"year": year, "quarter": quarter}
        return self._request(POST, TRADE_APPLY_ORDERS_FILLS_ARCHIVE, params)

    def get_fills_archive(self, year, quarter):
        params = {"year": year, "quarter": quarter}
        return self._request(GET, TRADE_RETRIEVE_ORDERS_FILLS_ARCHIVE, params)

    def get_easy_convert_currency_list(self):
        return self._request(GET, TRADE_EASY_CONVERT_CURRENCY_LIST)

    def easy_convert(self, fromCcy=[], toCcy=""):
        params = {"fromCcy": fromCcy, "toCcy": toCcy}
        return self._request(POST, TRADE_EASY_CONVERT, params)

    def get_easy_convert_history(self, before="", after="", limit=""):
        params = {"before": before, "after": after, "limit": limit}
        return self._request(GET, TRADE_CONVERT_EASY_HISTORY, params)

    def get_oneclick_repay_list(self, debtType=""):
        params = {"debtType": debtType}
        return self._request(GET, TRADE_ONE_CLICK_REPAY_CURRENCY_LIST, params)

    def oneclick_repay(self, debtCcy=[], repayCcy=""):
        params = {"debtCcy": debtCcy, "repayCcy": repayCcy}
        return self._request(POST, TRADE_ONE_CLICK_REPAY, params)

    def oneclick_repay_history(self, after="", before="", limit=""):
        params = {"after": after, "before": before, "limit": limit}
        return self._request(GET, TRADE_ONE_CLICK_REPAY_HISTORY, params)

    def cancel_all_orders(self, instType, instFamily):
        params = {"instType": instType, "instFamily": instFamily}
        return self._request(POST, TRADE_MASS_CANCEL, params)

    def cancel_all_after(self, timeOut):
        params = {"timeOut": timeOut}
        return self._request(POST, TRADE_CANCEL_ALL_AFTER, params)

    def get_account_rate_limit(self):
        return self._request(GET, TRADE_ACCOUNT_RATE_LIMIT)



import json

from .okxclient import OkxClient
from .consts import *


class TradeAPI(OkxClient):

    def __init__(self, api_key='-1', api_secret_key='-1', passphrase='-1', use_server_time=None, flag='1',
                 domain='https://www.okx.com', debug=False, proxy=None):
        OkxClient.__init__(self, api_key, api_secret_key, passphrase, use_server_time, flag, domain, debug, proxy)

    # Place Order
    def place_order(self, instId, tdMode, side, ordType, sz, ccy='', clOrdId='', tag='', posSide='', px='',
                    reduceOnly='', tgtCcy='', stpMode='', attachAlgoOrds=None, pxUsd='', pxVol='', banAmend=''):
        params = {'instId': instId, 'tdMode': tdMode, 'side': side, 'ordType': ordType, 'sz': sz, 'ccy': ccy,
                  'clOrdId': clOrdId, 'tag': tag, 'posSide': posSide, 'px': px, 'reduceOnly': reduceOnly,
                  'tgtCcy': tgtCcy, 'stpMode': stpMode, 'pxUsd': pxUsd, 'pxVol': pxVol, 'banAmend': banAmend}
        params['attachAlgoOrds'] = attachAlgoOrds
        return self._request_with_params(POST, PLACR_ORDER, params)

    # Place Multiple Orders
    def place_multiple_orders(self, orders_data):
        return self._request_with_params(POST, BATCH_ORDERS, orders_data)

    # Cancel Order
    def cancel_order(self, instId, ordId='', clOrdId=''):
        params = {'instId': instId, 'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(POST, CANCEL_ORDER, params)

    # Cancel Multiple Orders
    def cancel_multiple_orders(self, orders_data):
        return self._request_with_params(POST, CANCEL_BATCH_ORDERS, orders_data)

    # Amend Order
    def amend_order(self, instId, cxlOnFail='', ordId='', clOrdId='', reqId='', newSz='', newPx='', newTpTriggerPx='',
                    newTpOrdPx='', newSlTriggerPx='', newSlOrdPx='', newTpTriggerPxType='', newSlTriggerPxType='',
                    attachAlgoOrds='', newTriggerPx='', newOrdPx=''):
        params = {'instId': instId, 'cxlOnFail': cxlOnFail, 'ordId': ordId, 'clOrdId': clOrdId, 'reqId': reqId,
                  'newSz': newSz, 'newPx': newPx, 'newTpTriggerPx': newTpTriggerPx, 'newTpOrdPx': newTpOrdPx,
                  'newSlTriggerPx': newSlTriggerPx, 'newSlOrdPx': newSlOrdPx, 'newTpTriggerPxType': newTpTriggerPxType,
                  'newSlTriggerPxType': newSlTriggerPxType, 'newTriggerPx': newTriggerPx, 'newOrdPx': newOrdPx}
        params['attachAlgoOrds'] = attachAlgoOrds
        return self._request_with_params(POST, AMEND_ORDER, params)

    # Amend Multiple Orders
    def amend_multiple_orders(self, orders_data):
        return self._request_with_params(POST, AMEND_BATCH_ORDER, orders_data)

    # Close Positions
    def close_positions(self, instId, mgnMode, posSide='', ccy='', autoCxl='', clOrdId='', tag=''):
        params = {'instId': instId, 'mgnMode': mgnMode, 'posSide': posSide, 'ccy': ccy, 'autoCxl': autoCxl,
                  'clOrdId': clOrdId, 'tag': tag}
        return self._request_with_params(POST, CLOSE_POSITION, params)

    # Get Order Details
    def get_order(self, instId, ordId='', clOrdId=''):
        params = {'instId': instId, 'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(GET, ORDER_INFO, params)

    # Get Order List
    def get_order_list(self, instType='', uly='', instId='', ordType='', state='', after='', before='', limit='',
                       instFamily=''):
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'limit': limit, 'instFamily': instFamily}
        return self._request_with_params(GET, ORDERS_PENDING, params)

    # Get Order History (last 7 days）
    def get_orders_history(self, instType, uly='', instId='', ordType='', state='', after='', before='', begin='',
                           end='', limit='', instFamily=''):
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'begin': begin, 'end': end, 'limit': limit,
                  'instFamily': instFamily}
        return self._request_with_params(GET, ORDERS_HISTORY, params)

    # Get Order History (last 3 months)
    def get_orders_history_archive(self, instType, uly='', instId='', ordType='', state='', after='', before='',
                                   begin='', end='', limit='', instFamily=''):
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'begin': begin, 'end': end, 'limit': limit,
                  'instFamily': instFamily}
        return self._request_with_params(GET, ORDERS_HISTORY_ARCHIVE, params)

    # Get Transaction Details
    def get_fills(self, instType='', uly='', instId='', ordId='', after='', before='', limit='', instFamily='',begin='',end=''):
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordId': ordId, 'after': after, 'before': before,
                  'limit': limit, 'instFamily': instFamily,'begin': begin, 'end' :end}
        return self._request_with_params(GET, ORDER_FILLS, params)

    # Place Algo Order
    def place_algo_order(self, instId='', tdMode='', side='', ordType='', sz='', ccy='',
                         posSide='', reduceOnly='', tpTriggerPx='',
                         tpOrdPx='', slTriggerPx='', slOrdPx='',
                         triggerPx='', orderPx='', tgtCcy='', pxVar='',
                         pxSpread='',
                         szLimit='', pxLimit='', timeInterval='', tpTriggerPxType='', slTriggerPxType='',
                         callbackRatio='', callbackSpread='', activePx='', tag='', triggerPxType='', closeFraction=''
                         , quickMgnType='', algoClOrdId='', tradeQuoteCcy='', tpOrdKind='', cxlOnClosePos=''
                         , chaseType='', chaseVal='', maxChaseType='', maxChaseVal='', attachAlgoOrds=[]):
        params = {'instId': instId, 'tdMode': tdMode, 'side': side, 'ordType': ordType, 'sz': sz, 'ccy': ccy,
                  'posSide': posSide, 'reduceOnly': reduceOnly, 'tpTriggerPx': tpTriggerPx, 'tpOrdPx': tpOrdPx,
                  'slTriggerPx': slTriggerPx, 'slOrdPx': slOrdPx, 'triggerPx': triggerPx, 'orderPx': orderPx,
                  'tgtCcy': tgtCcy, 'pxVar': pxVar, 'szLimit': szLimit, 'pxLimit': pxLimit,
                  'timeInterval': timeInterval,
                  'pxSpread': pxSpread, 'tpTriggerPxType': tpTriggerPxType, 'slTriggerPxType': slTriggerPxType,
                  'callbackRatio': callbackRatio, 'callbackSpread': callbackSpread, 'activePx': activePx,
                  'tag': tag, 'triggerPxType': triggerPxType, 'closeFraction': closeFraction,
                  'quickMgnType': quickMgnType, 'algoClOrdId': algoClOrdId, 'tradeQuoteCcy': tradeQuoteCcy,
                  'tpOrdKind': tpOrdKind, 'cxlOnClosePos': cxlOnClosePos, 'chaseType': chaseType, 'chaseVal': chaseVal,
                  'maxChaseType': maxChaseType, 'maxChaseVal': maxChaseVal, 'attachAlgoOrds': attachAlgoOrds}
        return self._request_with_params(POST, PLACE_ALGO_ORDER, params)

    # Cancel Algo Order
    def cancel_algo_order(self, params):
        return self._request_with_params(POST, CANCEL_ALGOS, params)

    # Get Algo Order List
    def order_algos_list(self, ordType='', algoId='', instType='', instId='', after='', before='', limit=''):
        params = {'ordType': ordType, 'algoId': algoId, 'instType': instType, 'instId': instId, 'after': after,
                  'before': before, 'limit': limit}
        return self._request_with_params(GET, ORDERS_ALGO_PENDING, params)

    # Get Algo Order History
    def order_algos_history(self, ordType, state='', algoId='', instType='', instId='', after='', before='', limit=''):
        params = {'ordType': ordType, 'state': state, 'algoId': algoId, 'instType': instType, 'instId': instId,
                  'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, ORDERS_ALGO_HISTORY, params)

    # Get Transaction Details History
    def get_fills_history(self, instType, uly='', instId='', ordId='', after='', before='', limit='', instFamily=''):
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordId': ordId, 'after': after, 'before': before,
                  'limit': limit, 'instFamily': instFamily}
        return self._request_with_params(GET, ORDERS_FILLS_HISTORY, params)

    def get_easy_convert_currency_list(self):
        return self._request_without_params(GET, EASY_CONVERT_CURRENCY_LIST)

    def easy_convert(self, fromCcy=[], toCcy=''):
        params = {
            'fromCcy': fromCcy,
            'toCcy': toCcy
        }
        return self._request_with_params(POST, EASY_CONVERT, params)

    def get_easy_convert_history(self, before='', after='', limit=''):
        params = {
            'before': before,
            'after': after,
            'limit': limit
        }
        return self._request_with_params(GET, CONVERT_EASY_HISTORY, params)

    def get_oneclick_repay_list(self, debtType=''):
        params = {
            'debtType': debtType
        }
        return self._request_with_params(GET, ONE_CLICK_REPAY_SUPPORT, params)

    def oneclick_repay(self, debtCcy=[], repayCcy=''):
        params = {
            'debtCcy': debtCcy,
            'repayCcy': repayCcy
        }
        return self._request_with_params(POST, ONE_CLICK_REPAY, params)

    def oneclick_repay_history(self, after='', before='', limit=''):
        params = {
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, ONE_CLICK_REPAY_HISTORY, params)

    # Get algo order details
    def get_algo_order_details(self, algoId='', algoClOrdId=''):
        params = {'algoId': algoId, 'algoClOrdId': algoClOrdId}
        return self._request_with_params(GET, GET_ALGO_ORDER_DETAILS, params)

    # Amend algo order
    def amend_algo_order(self, instId='', algoId='', algoClOrdId='', cxlOnFail='', reqId='', newSz='',
                         newTpTriggerPx='', newTpOrdPx='', newSlTriggerPx='', newSlOrdPx='', newTpTriggerPxType='',
                         newSlTriggerPxType=''):
        params = {'instId': instId, 'algoId': algoId, 'algoClOrdId': algoClOrdId, 'cxlOnFail': cxlOnFail,
                  'reqId': reqId, 'newSz': newSz, 'newTpTriggerPx': newTpTriggerPx, 'newTpOrdPx': newTpOrdPx,
                  'newSlTriggerPx': newSlTriggerPx, 'newSlOrdPx': newSlOrdPx,
                  'newTpTriggerPxType': newTpTriggerPxType, 'newSlTriggerPxType': newSlTriggerPxType}
        return self._request_with_params(POST, AMEND_ALGO_ORDER, params)

    def get_oneclick_repay_list_v2(self):
        return self._request_without_params(GET, ONE_CLICK_REPAY_SUPPORT_V2)

    def oneclick_repay_v2(self, debtCcy='', repayCcyList=[]):
        params = {
            'debtCcy': debtCcy,
            'repayCcyList': repayCcyList
        }
        return self._request_with_params(POST, ONE_CLICK_REPAY_V2, params)

    def oneclick_repay_history_v2(self, after='', before='', limit=''):
        params = {
            'after': after,
            'before': before,
            'limit': limit
        }
        return self._request_with_params(GET, ONE_CLICK_REPAY_HISTORY_V2, params)
